################################################################################
#                                                                              #
# Copyright (C) 2003 Andrea Biardi                                             #
#                                                                              #
# This program is free software; you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation; either version 2 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
################################################################################

dnl ----------------------------------------------------------------------------
dnl
dnl Find everything we need to know in order to compile and link C/C++ programs;
dnl this includes finding preprocessors, compilers and linkers, testing if all
dnl works and tweaking some options to let the script go through.
dnl
dnl We need to wrap AC_PROG_CC and AC_PROG_CXX as they don't work with Borland's
dnl bcc32/bc++, which require the output filename to follow "-o", with no
dnl interleaving spaces; the same applies for the linker, with the addition that
dnl "-o" must be replaced by "-e". Given all this, why should we support it?
dnl Answer: because we bought it, we like it, and because it is much faster than
dnl gcc.
dnl
dnl Also, we need to pre-cache some settings, such as $ac_cv_c_inline.
dnl
dnl Please note that this macro isn't the safest thing: AC_PROG_CC and
dnl AC_PROG_CXX do a much better job by themselves, but we can't split them in
dnl pieces..
dnl
dnl This macro has been tested with various versions of gcc/g++, the Borland
dnl C/C++ 5.x compiler (bcc32), Borland's Kylix 3 (bc++) and the Intel 7.x/8.x
dnl C/C++ compiler (icc).
dnl
dnl Warning: never mix different C and C++ compilers..
dnl
dnl Output:
dnl   $CC                   = C compiler
dnl   $CXX                  = C++ compiler
dnl   $lt_ac_compiler       = "gnu"|"borland"|"intel"|"unknown"
dnl   $CPP                  = possibly rewritten
dnl   $CXXCPP               = possibly rewritten
dnl   $lt_compiler_o_option = possibly rewritten
dnl   $lt_linker_o_option   = possibly rewritten
dnl   $ac_cv_c_inline       = possibly rewritten
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_C_AND_CXX],[dnl

AC_MSG_CHECKING(for a C compiler)
if test x"$CC" = x; then
  for lt_compiler_candidate in gcc cc bcc32 bc++ icc
  do
    LT_AC_FIND_PROG(CC,$lt_compiler_candidate)
    if test x"$CC" = x; then
      :
    else
      break
    fi
  done
  if test x"$CC" = x; then
    AC_MSG_RESULT(not found)
    AC_MSG_ERROR(no acceptable C compiler found in PATH.)
  else
    AC_MSG_RESULT($CC)
  fi
else
  AC_MSG_RESULT(user-supplied ($CC))
fi

AC_LANG_PUSH(C)
AC_LANG_CONFTEST([AC_LANG_PROGRAM()])

for lt_op in '-o ' '-o'
do
  rm -f conftest.out
  if $CC ${lt_op}conftest.out -c conftest.$ac_ext > /dev/null 2>&1; then
    if test -f conftest.out; then
      lt_compiler_o_option=$lt_op
      break
    fi
  fi
done

for lt_op in '-o ' '-o' '-e ' '-e'
do
  rm -f conftest.out
  if $CC ${lt_op}conftest.out conftest.$ac_ext > /dev/null 2>&1; then
    if test -x conftest.out; then
      lt_linker_o_option=$lt_op
      break
    fi
  fi
done

AC_LANG_POP

rm -f .obj
rm -f .obj.obj

ac_cv_prog_CC=$CC
AC_PROG_CC

AC_MSG_CHECKING(for a C++ compiler)
if test x"$CXX" = x; then
  for lt_compiler_candidate in g++ c++ bcc32 bc++ icc
  do
    LT_AC_FIND_PROG(CXX,$lt_compiler_candidate)
    if test x"$CXX" = x; then
      :
    else
      break
    fi
  done
  if test x"$CXX" = x; then
    AC_MSG_RESULT(not found)
    AC_MSG_ERROR(no acceptable C++ compiler found in PATH.)
  else
    AC_MSG_RESULT($CXX)
  fi
else
  AC_MSG_RESULT(user-supplied ($CXX))
fi

ac_cv_prog_CXX=$CXX
AC_PROG_CXX

AC_LANG_PUSH(C++)

if test x"$ac_cv_c_compiler_gnu" = xyes && test x"$ac_cv_cxx_compiler_gnu" = xyes; then
  lt_ac_compiler=gnu
else
  lt_ac_compiler=
fi

dnl for Borland compilers, always reset lt_linker_o_option; -o is sometimes
dnl accepted, but seems to behave in a funny way.. (especially with Kylix)

AC_MSG_CHECKING(whether we are using the Borland C/C++ compiler)
AC_COMPILE_IFELSE([AC_LANG_PROGRAM([],[@%:@ifndef __BORLANDC__
  choke me
@%:@endif])],
                  lt_ac_compiler=borland
                  lt_linker_o_option='-e')
if test x"$lt_ac_compiler" = xborland; then
  AC_MSG_RESULT(yes)
else
  AC_MSG_RESULT(no)
fi

AC_MSG_CHECKING(whether we are using the Intel C/C++ compiler)
AC_COMPILE_IFELSE([AC_LANG_PROGRAM([],[@%:@ifndef __INTEL_COMPILER
  choke me
@%:@endif])],
                  lt_ac_compiler=intel)
if test x"$lt_ac_compiler" = xintel; then
  AC_MSG_RESULT(yes)
else
  AC_MSG_RESULT(no)
fi

AC_LANG_POP

case x$lt_ac_compiler in
  xborland)
    case $host in
      *-linux-*)
        test x"$CPP" = x && CPP=bcpp
        test x"$CXXCPP" = x && CXXCPP=bcpp
        ;;
      *)
        test x"$CPP" = x && CPP=cpp32
        test x"$CXXCPP" = x && CXXCPP=cpp32
        ;;
    esac
    ac_cv_c_inline=inline
    ;;
esac

AC_PROG_CPP
AC_PROG_CXXCPP

])

dnl ----------------------------------------------------------------------------
dnl
dnl Predefine CFLAGS to something useful, for a debug or production build, but
dnl without overriding what's already there, if set by the user.
dnl
dnl Input:
dnl   $lt_ac_compiler = see LT_AC_C_AND_CXX
dnl   $enable_debug   = "yes"|"no" (anything other than "no" means "yes")
dnl
dnl Output:
dnl   $CFLAGS         = C compiler flags
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_PREDEFINE_CFLAGS],[dnl
AC_MSG_CHECKING(which C compiler flags should be used)
if test x"$ac_env_CFLAGS_value" = x; then
  case $lt_ac_compiler in
    gnu)
      if test x"$enable_debug" = xno; then
        CFLAGS=$lt_ac_gnu_cflags_production
      else
        CFLAGS=$lt_ac_gnu_cflags_debug
      fi
      ;;
    borland)
      if test x"$enable_debug" = xno; then
        CFLAGS=$lt_ac_borland_cflags_production
      else
        CFLAGS=$lt_ac_borland_cflags_debug
      fi
      ;;
    intel)
      if test x"$enable_debug" = xno; then
        CFLAGS=$lt_ac_intel_cflags_production
      else
        CFLAGS=$lt_ac_intel_cflags_debug
      fi
      ;;
    *)
      if test x"$enable_debug" = xno; then
        CFLAGS=$lt_ac_unknown_cflags_production
      else
        CFLAGS=$lt_ac_unknown_cflags_debug
      fi
      ;;
  esac
  if test x"$CFLAGS" = x; then
    AC_MSG_RESULT(none)
  else
    AC_MSG_RESULT($CFLAGS)
  fi
else
  AC_MSG_RESULT(user-supplied ($CFLAGS))
fi])

dnl ----------------------------------------------------------------------------
dnl
dnl Predefine CXXFLAGS to something useful, for a debug or production build, but
dnl without overriding what's already there, if set by the user.
dnl
dnl Input:
dnl   $lt_ac_compiler = see LT_AC_C_AND_CXX
dnl   $enable_debug   = "yes"|"no" (anything other than "no" means "yes")
dnl
dnl Output:
dnl   $CXXFLAGS       = C++ compiler flags
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_PREDEFINE_CXXFLAGS],[dnl
AC_MSG_CHECKING(which C++ compiler flags should be used)
if test x"$ac_env_CXXFLAGS_value" = x; then
  case $lt_ac_compiler in
    gnu)
      if test x"$enable_debug" = xno; then
        CXXFLAGS=$lt_ac_gnu_cxxflags_production
      else
        CXXFLAGS=$lt_ac_gnu_cxxflags_debug
      fi
      ;;
    borland)
      if test x"$enable_debug" = xno; then
        CXXFLAGS=$lt_ac_borland_cxxflags_production
      else
        CXXFLAGS=$lt_ac_borland_cxxflags_debug
      fi
      ;;
    intel)
      if test x"$enable_debug" = xno; then
        CXXFLAGS=$lt_ac_intel_cxxflags_production
      else
        CXXFLAGS=$lt_ac_intel_cxxflags_debug
      fi
      ;;
    *)
      if test x"$enable_debug" = xno; then
        CXXFLAGS=$lt_ac_unknown_cxxflags_production
      else
        CXXFLAGS=$lt_ac_unknown_cxxflags_debug
      fi
      ;;
  esac
  if test x"$CXXFLAGS" = x; then
    AC_MSG_RESULT(none)
  else
    AC_MSG_RESULT($CXXFLAGS)
  fi
else
  AC_MSG_RESULT(user-supplied ($CXXFLAGS))
fi])

dnl ----------------------------------------------------------------------------
dnl
dnl Test if we can compile and link a simple C program, and halt the script
dnl otherwise.
dnl
dnl Input:
dnl   $CC      = C compiler
dnl   $CFLAGS  = compiler flags
dnl   $LDFLAGS = linker flags
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_TEST_C],[dnl

AC_MSG_CHECKING(whether we can compile and link a simple C program)

AC_LANG_PUSH(C)

lt_test_c_compiler_works=no
AC_COMPILE_IFELSE([AC_LANG_PROGRAM()],[lt_test_c_compiler_works=yes])

lt_test_c_linker_works=no
AC_LINK_IFELSE([AC_LANG_PROGRAM()],[lt_test_c_linker_works=yes])

AC_LANG_POP

case $lt_test_c_compiler_works,$lt_test_c_linker_works in
  yes,yes)
    AC_MSG_RESULT(yes)
    ;;
  yes,*)
    AC_MSG_RESULT(no)
    LT_AC_MSG_WARN([warning: we failed to link a simple C program; please check that you],
                   [         have a working C environment.])
    LT_AC_MSG_HINT([hint:    if your linker does work, the cause might be in the flags we],
                   [         used for this test; you may want to have a look at the log],
                   [         file to see what is really happening, and/or provide your],
                   [         own flag: try "$[]0 LDFLAGS=whatever".])
    AC_MSG_ERROR(cannot link C programs.)
    ;;
  *)
    AC_MSG_RESULT(no)
    LT_AC_MSG_WARN([warning: we failed to compile a simple C program; please check that],
                   [         you have a working C environment.])
    LT_AC_MSG_HINT([hint:    if your compiler does work, the cause might be in the flags],
                   [         we used for this test; you may want to have a look at the],
                   [         log file to see what is really happening, and/or provide],
                   [         your own flags: try "$[]0 CFLAGS=whatever".])
    AC_MSG_ERROR(cannot compile C programs.)
    ;;
esac

])

dnl ----------------------------------------------------------------------------
dnl
dnl Test if we can compile and link a simple C++ program, and halt the script
dnl otherwise.
dnl
dnl Input:
dnl   $CXX      = C compiler
dnl   $CXXFLAGS = compiler flags
dnl   $LDFLAGS  = linker flags
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_TEST_CXX],[dnl

AC_MSG_CHECKING(whether we can compile and link a simple C++ program)

AC_LANG_PUSH(C++)

lt_test_cxx_compiler_works=no
AC_COMPILE_IFELSE([AC_LANG_PROGRAM()],[lt_test_cxx_compiler_works=yes])

lt_test_cxx_linker_works=no
AC_LINK_IFELSE([AC_LANG_PROGRAM()],[lt_test_cxx_linker_works=yes])

AC_LANG_POP

case $lt_test_cxx_compiler_works,$lt_test_cxx_linker_works in
  yes,yes)
    AC_MSG_RESULT(yes)
    ;;
  yes,*)
    AC_MSG_RESULT(no)
    LT_AC_MSG_WARN([warning: we failed to link a simple C++ program; please check that],
                   [         you have a working C++ environment.])
    LT_AC_MSG_HINT([hint:    if your linker does work, the cause might be in the flags we],
                   [         used for this test; you may want to have a look at the log],
                   [         file to see what is really happening, and/or provide your],
                   [         own flag: try "$[]0 LDFLAGS=whatever".])
    AC_MSG_ERROR(cannot link C++ programs.)
    ;;
  *)
    AC_MSG_RESULT(no)
    LT_AC_MSG_WARN([warning: we failed to compile a simple C++ program; please check that],
                   [         you have a working C++ environment.])
    LT_AC_MSG_HINT([hint:    if your compiler does work, the cause might be in the flags],
                   [         we used for this test; you may want to have a look at the],
                   [         log file to see what is really happening, and/or provide],
                   [         your own flags: try "$[]0 CXXFLAGS=whatever".])
    AC_MSG_ERROR(cannot compile C++ programs.)
    ;;
esac

])

dnl ----------------------------------------------------------------------------
dnl
dnl Determine the size of a given C data type, such as "int" or "struct tm"; if
dnl the size cannot be determined (for example, when the data type is
dnl unsupported) 0 is reported, unless $3 is set to "yes", in which case the
dnl script is terminated.
dnl
dnl Input:
dnl   $1 = data type
dnl   $2 = additional prologue code
dnl   $3 = required? "yes"|"no" (anything other than "no" means "yes")
dnl
dnl Output:
dnl   $SIZEOF_$1 = size in bytes (*)
dnl (*) AC_SUBST'ed.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_SIZEOF],[dnl

AC_LANG_PUSH(C)

AC_MSG_CHECKING(the size of $1)
SIZEOF_[]AS_TR_CPP([$1])=
AC_LINK_IFELSE([AC_LANG_PROGRAM([dnl
@%:@include <stdio.h>
$2],[printf("%u\n",sizeof($1));])],
               [SIZEOF_[]AS_TR_CPP([$1])=`./conftest$ac_exeext 2>/dev/null`])

AC_LANG_POP

case x$SIZEOF_[]AS_TR_CPP([$1]) in
  x)
    if test x"$3" = xno; then
      AC_MSG_RESULT(not found (ignored))
      SIZEOF_[]AS_TR_CPP([$1])=0
    else
      AC_MSG_RESULT(error)
      AC_MSG_ERROR(cannot determine the size of $1.)
    fi
    ;;
  x1)
    AC_MSG_RESULT(1 byte)
    ;;
  *)
    AC_MSG_RESULT($SIZEOF_[]AS_TR_CPP([$1]) bytes)
    ;;
esac

AC_SUBST(SIZEOF_[]AS_TR_CPP([$1]))

])

dnl ----------------------------------------------------------------------------
dnl
dnl Define an AH_TEMPLATE for proper data alignment.
dnl
dnl Input:
dnl   $1 = round-up unit
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AH_DATA_ALIGNMENT],[dnl

LT_AC_TEMPLATE(]LT_AH_PREFIX[DATA_ALIGNMENT,This should reflect the correct data alignment for your machine; should be 4 for 32-bit machines)
LT_AC_DEFINE(]LT_AH_PREFIX[DATA_ALIGNMENT,[$1])

AH_BOTTOM([/* Round up a given size according to CXXBITS_DATA_ALIGNMENT. */
@%:@define ]LT_AH_PREFIX[ALIGN_SIZE(n) ((n+]LT_AH_PREFIX[DATA_ALIGNMENT-1) & ~(]LT_AH_PREFIX[DATA_ALIGNMENT-1))])])

dnl ----------------------------------------------------------------------------
dnl
dnl Determine a C integer type with a given minimum size; also, provide a
dnl configuration template for it.
dnl
dnl Input:
dnl   $1 = either "signed" or "unsigned"
dnl   $2 = data size, in bits (in this world, should be a multiple of 8)
dnl   $3 = additional prologue code
dnl   $4 = first data type to test for (don't specify signedness)
dnl   .. = more data types to test for (don't specify signedness)
dnl
dnl Output:
dnl   $INT$1 = data type (*)
dnl (*) $UINT$1 if requesting for an unsigned type
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_INT_TYPE],[dnl

LT_AC_TEMPLATE(LT_AH_PREFIX[]ifelse($1,unsigned,U)INT$2,Define to a $2-bit $1 integer type)

AC_LANG_PUSH(C)

AC_MSG_CHECKING(for a $2-bit $1 integer type)

ifelse($1,unsigned,U)INT$2=
LT_AC_LIST_CLEAR(lt_int_types)
LT_AC_LIST_APPEND(lt_int_types,m4_shift(m4_shift(m4_shift($@))))
LT_AC_LIST_WALK(lt_int_types,
                lt_int_type,
                [AC_LINK_IFELSE([AC_LANG_PROGRAM([@%:@include <stdio.h>
$3],[if (sizeof($1 $lt_int_type)*8 < $2) return 1;])],
                                [if ./conftest$ac_exeext 2>/dev/null; then ifelse($1,unsigned,U)INT$2=$lt_int_type; break; fi])])

AC_LANG_POP

case x$ifelse($1,unsigned,U)INT$2 in
  x)
    AC_MSG_RESULT(error)
    AC_MSG_ERROR(cannot determine an appropriate data type.)
    ;;
  *)
    AC_MSG_RESULT($ifelse($1,unsigned,U)INT$2)
    ;;
esac

LT_AC_DEFINE(LT_AH_PREFIX[]ifelse($1,unsigned,U)INT$2,ifelse($1,unsigned,unsigned) $ifelse($1,unsigned,U)INT$2)

])

dnl ----------------------------------------------------------------------------
dnl
dnl Check for a symbol in the std namespace; if the symbol cannot be found in
dnl std, but is present in the global namespace, define an AH_TEMPLATE so that
dnl it can be fixed (by the caller).
dnl
dnl Input:
dnl   $1 = function, such as "strchr()"
dnl   $2 = prologue code
dnl   $3 = main code (global namespace case)
dnl   $4 = main code (std namespace case)
dnl
dnl ----------------------------------------------------------------------------

AC_DEFUN([LT_AC_STD_OR_GLOBAL],[dnl

LT_AC_TEMPLATE(LT_AH_PREFIX[]GLOBAL_[]AS_TR_CPP(m4_translit([$1],())),Define if $1 is in the global namespace)

AC_LANG_PUSH(C++)

AC_MSG_CHECKING(for $1)

lt_result=no

AC_LINK_IFELSE([AC_LANG_PROGRAM([$2],
                                [$3])],
                                [lt_result=global])

AC_LINK_IFELSE([AC_LANG_PROGRAM([$2],
                                [$4])],
                                [lt_result=std])

AC_LANG_POP

case $lt_result in
  std)
    LT_AC_MSG_RESULT_USING(yes,namespace std)
    ;;
  global)
    LT_AC_MSG_RESULT_USING(yes,the global namespace)
    LT_AC_DEFINE(LT_AH_PREFIX[]GLOBAL_[]AS_TR_CPP(m4_translit([$1],())))
    ;;
  *)
    AC_MSG_RESULT(error)
    AC_MSG_ERROR(cannot find ::$1 or std::$1.)
    ;;
esac

])

dnl ----------------------------------------------------------------------------
dnl
dnl Test for common problems in the STL header cstdio.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_STL_CSTDIO],[dnl

LT_AC_STD_OR_GLOBAL([snprintf()],[dnl
@%:@include <cstdio>
@%:@include <stdio.h>],
                    [::snprintf(0,0,0,0);],
                    [std::snprintf(0,0,0,0);])

AH_BOTTOM([/* Bring additional symbols into namespace std (cstdio). */

@%:@ifdef ]LT_AH_PREFIX[GLOBAL_SNPRINTF
@%:@include <cstdio>
@%:@include <stdio.h>
namespace std { using ::snprintf; }
@%:@endif])

])

dnl ----------------------------------------------------------------------------
dnl
dnl Test for common problems in the STL header cstring.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_STL_CSTRING],[dnl

LT_AC_STD_OR_GLOBAL([strchr()],[dnl
@%:@include <cstring>
@%:@include <string.h>],
                    [::strchr((const char*)0,0);],
                    [std::strchr((const char*)0,0);])

LT_AC_STD_OR_GLOBAL([strrchr()],[dnl
@%:@include <cstring>
@%:@include <string.h>],
                    [::strrchr((const char*)0,0);],
                    [std::strrchr((const char*)0,0);])

AH_BOTTOM([/* Bring additional symbols into namespace std (cstring). */

@%:@ifdef ]LT_AH_PREFIX[GLOBAL_STRCHR
@%:@include <cstring>
@%:@include <string.h>
namespace std { using ::strchr; }
@%:@endif

@%:@ifdef ]LT_AH_PREFIX[GLOBAL_STRRCHR
@%:@include <cstring>
@%:@include <string.h>
namespace std { using ::strrchr; }
@%:@endif])

])

dnl ----------------------------------------------------------------------------
dnl
dnl Test for common problems in the STL header ctime.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_STL_CTIME],[dnl

LT_AC_STD_OR_GLOBAL([ctime()],[dnl
@%:@include <ctime>
@%:@include <time.h>],
                    [::ctime(0);],
                    [std::ctime(0);])

LT_AC_STD_OR_GLOBAL([localtime()],[dnl
@%:@include <ctime>
@%:@include <time.h>],
                    [::localtime(0);],
                    [std::localtime(0);])

LT_AC_STD_OR_GLOBAL([mktime()],[dnl
@%:@include <ctime>
@%:@include <time.h>],
                    [::mktime(0);],
                    [std::mktime(0);])

AH_BOTTOM([/* Bring additional symbols into namespace std (ctime). */

@%:@ifdef ]LT_AH_PREFIX[GLOBAL_CTIME
@%:@include <ctime>
@%:@include <time.h>
namespace std { using ::ctime; }
@%:@endif

@%:@ifdef ]LT_AH_PREFIX[GLOBAL_LOCALTIME
@%:@include <ctime>
@%:@include <time.h>
namespace std { using ::localtime; }
@%:@endif

@%:@ifdef ]LT_AH_PREFIX[GLOBAL_MKTIME
@%:@include <ctime>
@%:@include <time.h>
namespace std { using ::mktime; }
@%:@endif])

])
