################################################################################
#                                                                              #
# Copyright (C) 2003 Andrea Biardi                                             #
#                                                                              #
# This program is free software; you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation; either version 2 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
################################################################################

dnl ----------------------------------------------------------------------------
dnl
dnl Provide a "--enable-debug" option to choose between debug and production
dnl builds, the latter being the default.
dnl
dnl Also define an AH_TEMPLATE to disable assertions when building production
dnl code.
dnl
dnl Output:
dnl   $enable_debug = "yes"|"no" (*)
dnl (*) AC_SUBST'ed.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_ENABLE_DEBUG],[dnl

AC_ARG_ENABLE(debug,
              AC_HELP_STRING([--enable-debug],
                             [keep debugging information and don't optimize binaries] [[[no]]]),
              [],
              enable_debug=no)

AC_MSG_CHECKING(for build type)

case x$enable_debug in
  x)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_MISSING_ARG(--enable-debug)
    ;;
  xyes)
    AC_MSG_RESULT(debug)
    ;;
  xno)
    AC_MSG_RESULT(production)
    ;;
  *)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_BAD_ARG(--enable-debug)
    ;;
esac

AC_SUBST(enable_debug)

])

dnl ----------------------------------------------------------------------------
dnl
dnl Provide a configuration template so that we can insert C++ directives based
dnl on $enable_debug; by default, this macro defines ASSERT.
dnl
dnl Both DEBUG and ASSERT are prefixed with LT_AH_PREFIX.
dnl
dnl Input:
dnl   $1 = additional debug directives
dnl   $2 = additional production directives
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_CXX_DEBUG_TEMPLATE],[dnl

LT_AC_TEMPLATE(LT_AH_PREFIX[]DEBUG,Define in order to compile debug code)

AH_BOTTOM([/* Debug directives. */
@%:@ifdef ]LT_AH_PREFIX[DEBUG
@%:@define ]LT_AH_PREFIX[ASSERT(c) assert(c)]ifelse([$1],,,[
$1])[
@%:@else
@%:@define ]LT_AH_PREFIX[ASSERT(c)]ifelse([$2],,,[
$2])[
@%:@endif])

if test x"$enable_debug" = xyes; then
  LT_AC_DEFINE(LT_AH_PREFIX[]DEBUG)
fi

])
