################################################################################
#                                                                              #
# Copyright (C) 2003 Andrea Biardi                                             #
#                                                                              #
# This program is free software; you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation; either version 2 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
################################################################################

dnl ----------------------------------------------------------------------------
dnl
dnl Provide a "--enable-dependency-tracking" option to enable the creation and
dnl use of such files.
dnl
dnl Output:
dnl   $enable_dependency_tracking = "yes"|"no" (*)
dnl (*) AC_SUBST'ed.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_ENABLE_DEPENDENCY_TRACKING],[dnl

AC_ARG_ENABLE(dependency-tracking,
              AC_HELP_STRING([--enable-dependency-tracking],
                             [when disabled, speeds up one-time builds; enable this if you are going to tweak the source code] [[[no]]]),
              [],
              enable_dependency_tracking=no)

AC_MSG_CHECKING(whether to enable dependency tracking)

case x$enable_dependency_tracking in
  x)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_MISSING_ARG(--enable-dependency-tracking)
    ;;
  xyes|xno)
    AC_MSG_RESULT($enable_dependency_tracking)
    ;;
  *)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_BAD_ARG(--enable-dependency-tracking)
    ;;
esac

AC_SUBST(enable_dependency_tracking)

])

dnl ----------------------------------------------------------------------------
dnl
dnl Test how we can generate dependency files for C++ sources; this macro clears
dnl both $CXXCPP_M and $CXXCPP_M_OUT and does nothing else if
dnl $enable_dependency_tracking is not set to "yes".
dnl
dnl Output:
dnl   $CXXCPP_M     = suitable command, usually involving $CXXCPP (*)
dnl   $CXXCPP_M_OUT = "stdout" | "stderr" (*)
dnl (*) AC_SUBST'ed.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_CXX_DEPENDENCY_TRACKING],[dnl

CXXCPP_M=
CXXCPP_M_OUT=

if test x"$enable_dependency_tracking" = xyes; then

  AC_LANG_PUSH(C++)
  AC_MSG_CHECKING(how to generate C++ dependency files)

  if test x"$lt_ac_compiler" = xborland; then

    dnl Borland's preprocessor is almost unusable, as it insists on creating the
    dnl dependency file itself, and stubbornly refuses to write to stdout
    dnl (especially on windows)
    dnl
    dnl now, it also turns out that for some reason, the trick below won't work
    dnl with /dev/stdout; leave it like this, though it only works on linux

    case $host in
      *-linux-*)
        CXXCPP_M="$CXXCPP -m -md -mo/dev/stderr"
        CXXCPP_M_OUT=stderr
        ;;
    esac

  else

    LT_AC_LIST_CLEAR(lt_cxxcpp_m_candidates)
    LT_AC_LIST_APPEND(lt_cxxcpp_m_candidates,$CXXCPP -M)
    LT_AC_LIST_APPEND(lt_cxxcpp_m_candidates,$CXX -M)

    AC_LANG_CONFTEST([AC_LANG_PROGRAM([@%:@include <iostream>])])

    LT_AC_LIST_LOG(lt_cxxcpp_m_candidates,candidate commands)
    LT_AC_LIST_WALK(lt_cxxcpp_m_candidates,
                    lt_cxxcpp_m_candidate,
                    [if $lt_cxxcpp_m_candidate $CPPFLAGS conftest.$ac_ext 2>/dev/null | grep "^conftest\.$ac_objext: conftest\.$ac_ext" > /dev/null; then
                      CXXCPP_M=$lt_cxxcpp_m_candidate
                      CXXCPP_M_OUT=stdout
                      break
                    fi])

  fi

  if test x"$CXXCPP_M" = x; then
    AC_MSG_RESULT(error)
    AC_MSG_ERROR(don't know how to generate C++ dependency files.)
  else
    LT_AC_MSG_RESULT_USING(ok,$CXXCPP_M on $CXXCPP_M_OUT)
  fi
  AC_LANG_POP

fi

AC_SUBST(CXXCPP_M)

])
