################################################################################
#                                                                              #
# Copyright (C) 2003 Andrea Biardi                                             #
#                                                                              #
# This program is free software; you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation; either version 2 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
################################################################################

dnl ----------------------------------------------------------------------------
dnl
dnl Provide a "--with-doc" option to choose one or more formats for the (LaTeX)
dnl documentation. The default is to build all formats for which the user has
dnl appropriate tools.
dnl
dnl Output:
dnl   $with_doc     = see below (*1) (*3)
dnl   $with_doc_dvi = "yes"|"no" (*3)
dnl   $with_doc_ps  = "yes"|"no" (*3)
dnl   $with_doc_pdf = "yes"|"no" (*3)
dnl   $LATEX        = path to LaTeX (*2) (*3)
dnl   $DVIPS        = path to dvips (*2) (*3)
dnl   $DVIPDFM      = path to dvipdfm (*2) (*3)
dnl (*1) either empty or a description of choosen formats.
dnl (*2) empty string if the tool is not required, or not found.
dnl (*3) AC_SUBST'ed.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_WITH_DOC],[dnl

AC_ARG_WITH(doc,
            AC_HELP_STRING([--with-doc],
                           [build the documentation, in one or more formats; multiple choices can be separated by commas (yes|no|dvi|ps|pdf)] [[[auto]]]),
            [],
            with_doc=auto)

AC_MSG_CHECKING(whether to build the documentation)

case x$with_doc in
  x)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_MISSING_ARG(--with-doc)
    ;;
  xyes|xauto)
    with_doc_dvi=$with_doc
    with_doc_ps=$with_doc
    with_doc_pdf=$with_doc
    ;;
  *)
    with_doc_dvi=no
    with_doc_ps=no
    with_doc_pdf=no
    if test x"$with_doc" = xno; then
      :
    else
      lt_doc_save_ifs=$IFS
      IFS=,
      for lt_doc_format in $with_doc
      do
        case $lt_doc_format in
          dvi)
            with_doc_dvi=yes
            ;;
          ps)
            with_doc_dvi=yes
            with_doc_ps=yes
            ;;
          pdf)
            with_doc_dvi=yes
            with_doc_pdf=yes
            ;;
          *)
            IFS=$lt_doc_save_ifs
            AC_MSG_RESULT(error)
            LT_AC_MSG_ERROR_BAD_ARG(--with-doc)
            ;;
        esac
      done
      IFS=$lt_doc_save_ifs
    fi
    ;;
esac

if test x"$with_doc_dvi" = xno; then
  :
else
  LT_AC_FIND_PROG(LATEX,latex)
  if test x"$LATEX" = x; then
    if test x"$with_doc_dvi" = xyes; then
      AC_MSG_RESULT(error)
      AC_MSG_ERROR(cannot find LaTeX.)
    fi
    with_doc_dvi=no
    with_doc_ps=no
    with_doc_pdf=no
  else
    with_doc_dvi=yes
  fi
fi

if test x"$with_doc_ps" = xno; then
  :
else
  LT_AC_FIND_PROG(DVIPS,dvips)
  if test x"$DVIPS" = x; then
    if test x"$with_doc_ps" = xyes; then
      AC_MSG_RESULT(error)
      AC_MSG_ERROR(cannot find dvips.)
    fi
    with_doc_ps=no
  else
    with_doc_ps=yes
  fi
fi

if test x"$with_doc_pdf" = xno; then
  :
else
  LT_AC_FIND_PROG(DVIPDFM,dvipdfm)
  if test x"$DVIPDFM" = x; then
    if test x"$with_doc_pdf" = xyes; then
      AC_MSG_RESULT(error)
      AC_MSG_ERROR(cannot find dvipdfm.)
    fi
    with_doc_pdf=no
  else
    with_doc_pdf=yes
  fi
fi

case $with_doc_dvi,$with_doc_ps,$with_doc_pdf in
  yes,yes,yes)
    with_doc="DVI, PostScript and PDF"
    ;;
  yes,yes,no)
    with_doc="DVI and PostScript"
    ;;
  yes,no,yes)
    with_doc="DVI and PDF"
    ;;
  yes,no,no)
    with_doc="DVI only"
    ;;
  *)
    with_doc=
    ;;
esac

if test x"$with_doc" = x; then
  AC_MSG_RESULT(no)
else
  AC_MSG_RESULT($with_doc)
fi

AC_SUBST(with_doc)
AC_SUBST(with_doc_dvi)
AC_SUBST(with_doc_ps)
AC_SUBST(with_doc_pdf)
AC_SUBST(LATEX)
AC_SUBST(DVIPS)
AC_SUBST(DVIPDFM)

])

dnl ----------------------------------------------------------------------------
dnl
dnl Provide a "--with-doc-paper" option to choose between "a4" and "letter"
dnl paper sizes for the docs, the former being the default.
dnl
dnl Output:
dnl   $with_doc_paper = "a4"|"letter" (*)
dnl (*) AC_SUBST'ed.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_WITH_DOC_PAPER],[dnl

AC_ARG_WITH(doc-paper,
            AC_HELP_STRING([--with-doc-paper],
                           [paper size for the documentation (a4|letter)] [[[a4]]]),
            [],
            with_doc_paper=a4)

AC_MSG_CHECKING(for paper size)

case x$with_doc_paper in
  x)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_MISSING_ARG(--with-doc-paper)
    ;;
  xa4|xletter)
    AC_MSG_RESULT($with_doc_paper)
    ;;
  *)
    AC_MSG_RESULT(error)
    if test x"$with_doc_paper" = xno; then
      LT_AC_MSG_HINT([hint:    if you really want to save trees, don't print.])
    fi
    LT_AC_MSG_ERROR_BAD_ARG(--with-doc-paper)
    ;;
esac

AC_SUBST(with_doc_paper)

])

dnl ----------------------------------------------------------------------------
dnl
dnl Provide a "--enable-doc-twoside" option to enable two-sided documentation
dnl printout; default is one-sided printout.
dnl
dnl Output:
dnl   $enable_doc_twoside = "yes"|"no" (*)
dnl   $doc_printout       = "oneside"|"twoside" (*)
dnl (*) AC_SUBST'ed.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_ENABLE_DOC_TWOSIDE],[dnl

AC_ARG_ENABLE(doc-twoside,
              AC_HELP_STRING([--enable-doc-twoside],
                             [create two-sided documentation printout] [[[no]]]),
              [],
              enable_doc_twoside=no)

AC_MSG_CHECKING(for two-sided printout)

case x$enable_doc_twoside in
  x)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_MISSING_ARG(--enable-doc-twoside)
    ;;
  xyes|xno)
    AC_MSG_RESULT($enable_doc_twoside)
    if test x"$enable_doc_twoside" = xyes; then
      doc_printout=twoside
    else
      doc_printout=oneside
    fi
    ;;
  *)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_BAD_ARG(--enable-doc-twoside)
    ;;
esac

AC_SUBST(enable_doc_twoside)
AC_SUBST(doc_printout)

])
