################################################################################
#                                                                              #
# Copyright (C) 2003 Andrea Biardi                                             #
#                                                                              #
# This program is free software; you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation; either version 2 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
################################################################################

dnl ----------------------------------------------------------------------------
dnl
dnl Provide a "--with-interbase" option, and the relevant checks for a suitable
dnl library, with C++ linkage.
dnl
dnl Input:
dnl   $require_interbase  = "yes"|"no" (anything other than "no" means "yes")
dnl   $CXX                = C++ compiler
dnl   $CPPFLAGS           = standard preprocessor flags
dnl   $CXXFLAGS           = standard compiler flags
dnl   $LDFLAGS            = standard linker flags
dnl   $INTERBASE          = path to Interbase (optional)
dnl
dnl Output:
dnl   $with_interbase     = "yes"|"no" (*1)
dnl   $cppflags_interbase = preprocessor flags (*1) (*2)
dnl   $ldflags_interbase  = linker flags (*1) (*2)
dnl   HAVE_INTERBASE      = preprocessor symbol
dnl (*1) AC_SUBST'ed.
dnl (*2) empty string if $with_interbase is set to "no".
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_WITH_INTERBASE],[dnl

AC_ARG_WITH(interbase,
            AC_HELP_STRING([--with-interbase],
                           [path to Interbase/Firebird (yes|no|DIR)] [[[auto]]]),
            [],
            with_interbase=auto)

AC_ARG_VAR(INTERBASE,path to Interbase)
LT_AC_VAR_CHECKING(INTERBASE)

AC_ARG_VAR(FIREBIRD,path to Firebird)
LT_AC_VAR_CHECKING(FIREBIRD)

LT_AC_TEMPLATE_HAVE(interbase,the Interbase/Firebird library)

LT_AC_INITIALIZE_WITH_LIBRARY(interbase)
AC_LANG_PUSH(C++)

# check for headers

AC_MSG_CHECKING(for Interbase/Firebird headers)
lt_have_header=no

if test x"$with_interbase" = xno; then

  # disabled

  AC_MSG_RESULT(disabled)

else

  # prepare a list of candidates

  LT_AC_LIST_CLEAR(lt_cppflags_candidates)

  # user-supplied location, if any

  if test x"$with_interbase" = xyes; then
    :
  elif test x"$with_interbase" = xauto; then
    :
  else
    LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,$with_interbase)
    LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,$with_interbase/include)
    case $host in
      *-*-mingw32)
        LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,$with_interbase/sdk/include)
        ;;
      *-*-darwin*)
        LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,$with_interbase/Headers)
        ;;
    esac
  fi

  # settings from the environment

  if test x"$INTERBASE" = x; then
    :
  else
    LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,$INTERBASE)
    LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,$INTERBASE/include)
    case $host in
      *-*-mingw32)
        LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,$INTERBASE/sdk/include)
        ;;
    esac
  fi

  if test x"$FIREBIRD" = x; then
    :
  else
    LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,$FIREBIRD)
    LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,$FIREBIRD/include)
    case $host in
      *-*-darwin*)
        LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,$FIREBIRD/Headers)
        ;;
    esac
  fi

  # platform-independent defaults

  LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates)
  LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,/usr/include)
  LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,/usr/local/include)
  LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,/opt/interbase/include)
  LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,/opt/firebird/include)

  # platform-dependent defaults

  case $host in
    *-*-mingw32)
      LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,$lt_prgfiles_dir/borland/interbase/sdk/include)
      LT_AC_APPEND_CPPFLAGS_STD(lt_cppflags_candidates,$lt_prgfiles_dir/firebird/firebird_1_5/include)
      ;;
    *-*-darwin*)
      LT_AC_APPEND_CPPFLAGS_FMK(lt_cppflags_candidates,Firebird)
      ;;
  esac

  # loop over all candidates

  LT_AC_LIST_LOG(lt_cppflags_candidates,candidate preprocessor flags)
  LT_AC_LIST_WALK(lt_cppflags_candidates,
                  lt_cppflags_candidate,
                  [LT_AC_REWRITE_AC_COMPILE_CXX($lt_cppflags_candidate)
                  LT_AC_LOG_CXX_COMPILER_SETTINGS
                  AC_COMPILE_IFELSE([AC_LANG_PROGRAM([@%:@include <ibase.h>],
                                                     [[ISC_STATUS v[20]; isc_db_handle h; isc_detach_database(v,&h);]])],
                                    [lt_have_header=yes; cppflags_interbase=$lt_cppflags_candidate; break])])
  LT_AC_MSG_RESULT_USING($lt_have_header,$cppflags_interbase)

fi

# check for library

AC_MSG_CHECKING(for Interbase/Firebird library)
lt_have_lib=no

if test x"$with_interbase" = xno; then

  # disabled

  AC_MSG_RESULT(disabled)

else

  # prepare a list of candidates

  LT_AC_LIST_CLEAR(lt_ldflags_candidates)

  # user-supplied location, if any

  if test x"$with_interbase" = x; then
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_MISSING_ARG(--with-interbase)
  elif test x"$with_interbase" = xyes; then
    :
  elif test x"$with_interbase" = xauto; then
    :
  else
    LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,gds,$with_interbase)
    LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,gds,$with_interbase/lib)
    LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,gds,$with_interbase/bin)
    LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,fbclient,$with_interbase)
    LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,fbclient,$with_interbase/lib)
    LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,fbclient,$with_interbase/bin)
    case $host in
      *-*-mingw32)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32,$with_interbase)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32,$with_interbase/bin)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32,$with_interbase/sdk/lib)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32_ms,$with_interbase)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32_ms,$with_interbase/sdk/lib_ms)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient,$with_interbase)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient,$with_interbase/bin)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient_bor,$with_interbase)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient_bor,$with_interbase/lib)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient_ms,$with_interbase)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient_ms,$with_interbase/lib)
        ;;
      *-*-darwin*)
        LT_AC_APPEND_LDFLAGS_FMK(lt_ldflags_candidates,Firebird,$with_interbase)
        ;;
    esac
  fi

  # settings from the environment

  if test x"$INTERBASE" = x; then
    :
  else
    LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,gds,$INTERBASE)
    LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,gds,$INTERBASE/lib)
    LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,gds,$INTERBASE/bin)
    case $host in
      *-*-mingw32)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32,$INTERBASE)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32,$INTERBASE/bin)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32,$INTERBASE/sdk/lib)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32_ms,$INTERBASE)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32_ms,$INTERBASE/sdk/lib_ms)
        ;;
    esac
  fi

  if test x"$FIREBIRD" = x; then
    :
  else
    LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,fbclient,$FIREBIRD)
    LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,fbclient,$FIREBIRD/lib)
    LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,fbclient,$FIREBIRD/bin)
    case $host in
      *-*-mingw32)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32,$FIREBIRD)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32,$FIREBIRD/bin)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient,$FIREBIRD)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient,$FIREBIRD/bin)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient_bor,$FIREBIRD)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient_bor,$FIREBIRD/lib)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient_ms,$FIREBIRD)
        LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient_ms,$FIREBIRD/lib)
        ;;
    esac
  fi

  # platform-independent defaults

  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,,/lib)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,,/usr/lib)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,,/usr/local/lib)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,gds)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,gds,/lib)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,gds,/usr/lib)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,gds,/usr/local/lib)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,gds,/opt/interbase/lib)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,gds,/opt/interbase/bin)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,fbclient)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,fbclient,/lib)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,fbclient,/usr/lib)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,fbclient,/usr/local/lib)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,fbclient,/opt/firebird/lib)
  LT_AC_APPEND_LDFLAGS_STD(lt_ldflags_candidates,fbclient,/opt/firebird/bin)

  # platform-dependent defaults

  case $host in
    *-*-mingw32)
      LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32)
      LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32,$lt_prgfiles_dir/borland/interbase/bin)
      LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32,$lt_prgfiles_dir/borland/interbase/sdk/lib)
      LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32_ms,$lt_prgfiles_dir/borland/interbase/sdk/lib_ms)
      LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient)
      LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,gds32,$lt_prgfiles_dir/firebird/firebird_1_5/bin)
      LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient,$lt_prgfiles_dir/firebird/firebird_1_5/bin)
      LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient_bor,$lt_prgfiles_dir/firebird/firebird_1_5/lib)
      LT_AC_APPEND_LDFLAGS_WIN(lt_ldflags_candidates,fbclient_ms,$lt_prgfiles_dir/firebird/firebird_1_5/lib)
      ;;
    *-*-darwin*)
      LT_AC_APPEND_LDFLAGS_FMK(lt_ldflags_candidates,Firebird)
      ;;
  esac

  # loop over all candidates

  LT_AC_LIST_LOG(lt_ldflags_candidates,candidate linker flags)
  LT_AC_LIST_WALK(lt_ldflags_candidates,
                  lt_ldflags_candidate,
                  [LT_AC_REWRITE_AC_LINK_CXX($cppflags_interbase,$lt_ldflags_candidate)
                  LT_AC_LOG_CXX_LINKER_SETTINGS
                  AC_LINK_IFELSE([AC_LANG_PROGRAM([@%:@include <ibase.h>],
                                                  [isc_detach_database(0,0);])],
                                 [lt_have_lib=yes; ldflags_interbase=$lt_ldflags_candidate; break])])
  LT_AC_MSG_RESULT_USING($lt_have_lib,$ldflags_interbase)

fi

AC_LANG_POP
LT_AC_FINALIZE_WITH_LIBRARY(interbase)

AC_SUBST(with_interbase)
AC_SUBST(cppflags_interbase)
AC_SUBST(ldflags_interbase)

if test x"$with_interbase" = xyes; then
  LT_AC_HAVE(interbase)
fi

])

dnl ----------------------------------------------------------------------------
dnl
dnl Provide a "--enable-sql-error-msgs" option to choose the flavor of error
dnl messages.
dnl
dnl Output:
dnl   $enable_sql_error_msgs = "yes"|"no"
dnl   IBXX_ERROR_MSGS_SQL    = preprocessor symbol
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_ENABLE_SQL_ERROR_MSGS],[dnl

AC_ARG_ENABLE(sql-error-msgs,
              AC_HELP_STRING([--enable-sql-error-msgs],
                             [use SQL-style error messages] [[[no]]]),
              [],
              enable_sql_error_msgs=no)

LT_AC_TEMPLATE(IBXX_ERROR_MSGS_SQL,[You can choose between two flavors of error messages; normally, classic Interbase-style error messages give more information, but you can define the following symbol to toggle this behavior])

AC_MSG_CHECKING(whether you want SQL-style error messages)

case x$enable_sql_error_msgs in
  x)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_MISSING_ARG(--enable-sql-error-msgs)
    ;;
  xyes)
    AC_MSG_RESULT(SQL-style)
    LT_AC_DEFINE(IBXX_ERROR_MSGS_SQL)
    ;;
  xno)
    AC_MSG_RESULT(Interbase-style)
    ;;
  *)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_BAD_ARG(--enable-sql-error-msgs)
    ;;
esac])

dnl ----------------------------------------------------------------------------
dnl
dnl Provide a "--with-default-dialect" option to specify the default dialect.
dnl
dnl Output:
dnl   $with_default_dialect = "1"|"3"
dnl   IBXX_DEFAULT_DIALECT  = preprocessor symbol
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_WITH_DEFAULT_DIALECT],[dnl

AC_ARG_WITH(default-dialect,
            AC_HELP_STRING([--with-default-dialect],
                           [default database dialect (1|3)] [[[3]]]),
            [],
            with_default_dialect=3)

LT_AC_TEMPLATE(IBXX_DEFAULT_DIALECT,There's full support for both dialect 1 and dialect 3; choose the one you would like to use by default)

AC_MSG_CHECKING(for default database dialect)

case x$with_default_dialect in
  x)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_MISSING_ARG(--with-default-dialect)
    ;;
  x1|x3)
    AC_MSG_RESULT($with_default_dialect)
    LT_AC_DEFINE(IBXX_DEFAULT_DIALECT,$with_default_dialect)
    ;;
  *)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_BAD_ARG(--with-default-dialect)
    ;;
esac])
