################################################################################
#                                                                              #
# Copyright (C) 2003 Andrea Biardi                                             #
#                                                                              #
# This program is free software; you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation; either version 2 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
################################################################################

dnl ----------------------------------------------------------------------------
dnl
dnl Append preprocessor flags for a given library to an existing list of
dnl candidates.
dnl
dnl Input:
dnl   $1 = list variable name
dnl   $2 = path (may be empty)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_APPEND_CPPFLAGS_STD],[dnl
ifelse([$2],,
       [LT_AC_LIST_APPEND([$1])],
       [lt_append_cppflags_std=$2
       LT_AC_VAR_QUOTE(lt_append_cppflags_std)
       LT_AC_LIST_APPEND([$1],-I$lt_append_cppflags_std)])
])

dnl ----------------------------------------------------------------------------
dnl
dnl Append preprocessor flags for a given MacOS-X framework to an existing list
dnl of candidates.
dnl
dnl Input:
dnl   $1 = list variable name
dnl   $2 = framework name
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_APPEND_CPPFLAGS_FMK],[dnl

lt_append_cppflags_fmk=$2
LT_AC_VAR_QUOTE(lt_append_cppflags_fmk)
LT_AC_LIST_APPEND([$1],-F$lt_append_cppflags_fmk)

LT_AC_APPEND_CPPFLAGS_STD([$1],[/System/Library/Frameworks/$2.framework/Headers])
LT_AC_APPEND_CPPFLAGS_STD([$1],[/Library/Frameworks/$2.framework/Headers])
LT_AC_APPEND_CPPFLAGS_STD([$1],[/Local/Library/Frameworks/$2.framework/Headers])

])

dnl ----------------------------------------------------------------------------
dnl
dnl Append linker flags for a given library to an existing list of candidates.
dnl
dnl Input:
dnl   $1 = list variable name
dnl   $2 = library name (may be empty)
dnl   $3 = path (may be empty)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_APPEND_LDFLAGS_STD],[dnl

dnl under linux, when using a Borland compiler, first try with just the library
dnl name (and path, if given)

case $host in
  *-linux-*)
    if test x"$lt_ac_compiler" = xborland; then
      ifelse([$2],,
             [lt_append_ldflags_std_lib=],
             [lt_append_ldflags_std_lib=$2
             LT_AC_VAR_QUOTE(lt_append_ldflags_std_lib)
             lt_append_ldflags_std_lib=lib$lt_append_ldflags_std_lib.so])
      ifelse([$3],,
             [lt_append_ldflags_std_dir=],
             [lt_append_ldflags_std_dir=$3
             LT_AC_VAR_QUOTE(lt_append_ldflags_std_dir)
             lt_append_ldflags_std_dir=-L$lt_append_ldflags_std_dir])
      if test x"$lt_append_ldflags_std_lib" = x || test x"$lt_append_ldflags_std_dir" = x; then
        :
      else
        lt_append_ldflags_std_lib=" $lt_append_ldflags_std_lib"
      fi
      LT_AC_LIST_APPEND([$1],$lt_append_ldflags_std_dir$lt_append_ldflags_std_lib)
    fi
    ;;
esac

dnl "normal" case

ifelse([$2],,
       [lt_append_ldflags_std_lib=],
       [lt_append_ldflags_std_lib=$2
       LT_AC_VAR_QUOTE(lt_append_ldflags_std_lib)
       lt_append_ldflags_std_lib=-l$lt_append_ldflags_std_lib])

ifelse([$3],,
       [lt_append_ldflags_std_dir=],
       [lt_append_ldflags_std_dir=$3
       LT_AC_VAR_QUOTE(lt_append_ldflags_std_dir)
       lt_append_ldflags_std_dir=-L$lt_append_ldflags_std_dir])

if test x"$lt_append_ldflags_std_lib" = x || test x"$lt_append_ldflags_std_dir" = x; then
  :
else
  lt_append_ldflags_std_lib=" $lt_append_ldflags_std_lib"
fi
LT_AC_LIST_APPEND([$1],$lt_append_ldflags_std_dir$lt_append_ldflags_std_lib)

dnl for linux, also try with -ldl

case $host in
  *-linux-*)
    lt_append_ldflags_std_dl=$lt_append_ldflags_std_dir$lt_append_ldflags_std_lib;
    if test x"$lt_append_ldflags_std_dl" = x; then
      lt_append_ldflags_std_dl="-ldl"
    else
      lt_append_ldflags_std_dl="$lt_append_ldflags_std_dl -ldl"
    fi
    LT_AC_LIST_APPEND([$1],$lt_append_ldflags_std_dl)
    ;;
esac

])

dnl ----------------------------------------------------------------------------
dnl
dnl Append linker flags for a given MS Windows library to an existing list of
dnl candidates.
dnl
dnl Input:
dnl   $1 = list variable name
dnl   $2 = library name
dnl   $3 = path (if empty, search for candidates in $PATH)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_APPEND_LDFLAGS_WIN],[dnl
ifelse([$3],,
       [LT_AC_FIND_FILE([$2.dll],,,
                        [lt_append_ldflags_win=$lt_find_file_candidate/$2.dll
                        LT_AC_VAR_QUOTE(lt_append_ldflags_win)
                        LT_AC_LIST_APPEND([$1],$lt_append_ldflags_win)])
       LT_AC_FIND_FILE([$2.lib],,,
                       [lt_append_ldflags_win=$lt_find_file_candidate/$2.lib
                       LT_AC_VAR_QUOTE(lt_append_ldflags_win)
                       LT_AC_LIST_APPEND([$1],$lt_append_ldflags_win)])],
       [lt_append_ldflags_win=$3/$2.dll
       LT_AC_VAR_QUOTE(lt_append_ldflags_win)
       LT_AC_LIST_APPEND([$1],$lt_append_ldflags_win)
       lt_append_ldflags_win=$3/$2.lib
       LT_AC_VAR_QUOTE(lt_append_ldflags_win)
       LT_AC_LIST_APPEND([$1],$lt_append_ldflags_win)])
])

dnl ----------------------------------------------------------------------------
dnl
dnl Append linker flags for a given MacOS-X framework to an existing list of
dnl candidates.
dnl
dnl Input:
dnl   $1 = list variable name
dnl   $2 = framework name (may be empty)
dnl   $3 = path (may be empty)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_APPEND_LDFLAGS_FMK],[dnl

ifelse([$2],,
       [lt_append_ldflags_fmk_lib=],
       [lt_append_ldflags_fmk_lib=$2
       LT_AC_VAR_QUOTE(lt_append_ldflags_fmk_lib)
       lt_append_ldflags_fmk_lib="-framework $lt_append_ldflags_fmk_lib"])

ifelse([$3],,
       [lt_append_ldflags_fmk_dir=],
       [lt_append_ldflags_fmk_dir=$3
       LT_AC_VAR_QUOTE(lt_append_ldflags_fmk_dir)
       lt_append_ldflags_fmk_dir=-F$lt_append_ldflags_fmk_dir])

if test x"$lt_append_ldflags_fmk_lib" = x || test x"$lt_append_ldflags_fmk_dir" = x; then
  :
else
  lt_append_ldflags_fmk_lib=" $lt_append_ldflags_fmk_lib"
fi
LT_AC_LIST_APPEND([$1],$lt_append_ldflags_fmk_dir$lt_append_ldflags_fmk_lib)

])
