################################################################################
#                                                                              #
# Copyright (C) 2003 Andrea Biardi                                             #
#                                                                              #
# This program is free software; you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation; either version 2 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
################################################################################

dnl ----------------------------------------------------------------------------
dnl
dnl Setup things.
dnl
dnl First, call AC_CANONICAL_{BUILD,HOST} to determine platform. After that,
dnl $host will contain the canonical name of the system configure is running on,
dnl and $build will contain the canonical name of the system the user is
dnl configuring for. Say "thanks" to the GNU folks for this.
dnl
dnl Immediately define $lt_compiler_o_option and $lt_linker_o_option to be on
dnl the safe side (see lt_ac_c_and_cxx.m4 for details on this one).
dnl
dnl Finally, run some trivial initialization code (see also the very bottom of
dnl this file).
dnl
dnl ----------------------------------------------------------------------------
AC_CANONICAL_BUILD
AC_CANONICAL_HOST
lt_newline='
'
lt_digits=0123456789
lt_locase_chars=abcdefghijklmnopqrstuvwxyz
lt_upcase_chars=ABCDEFGHIJKLMNOPQRSTUVWXYZ
lt_ac_has_hints=no
lt_ac_has_warns=no
lt_compiler_o_option='-o '
lt_linker_o_option='-o '
lt_ac_gnu_cflags_debug="-pipe -g -O0 -W -Wall"
lt_ac_gnu_cflags_production="-pipe -O3 -funroll-loops"
lt_ac_gnu_cxxflags_debug=$lt_ac_gnu_cflags_debug
lt_ac_gnu_cxxflags_production=$lt_ac_gnu_cflags_production
lt_ac_borland_cflags_debug="-Od -Hu -k -lv -r- -v -vi- -w -w-8026 -w-8027 -X -xp -xs -y"
lt_ac_borland_cflags_production="-O2 -d -Hu -k- -Ve -Vx -v- -vi -w-8026 -w-8027 -X"
lt_ac_borland_cxxflags_debug=$lt_ac_borland_cflags_debug
lt_ac_borland_cxxflags_production=$lt_ac_borland_cflags_production
lt_ac_intel_cflags_debug="-g -O0 -i_dynamic -Wall -wd279,383,810,981"
lt_ac_intel_cflags_production="-O3 -ip -i_dynamic"
lt_ac_intel_cxxflags_debug=$lt_ac_intel_cflags_debug
lt_ac_intel_cxxflags_production=$lt_ac_intel_cflags_production
lt_ac_unknown_cflags_debug=
lt_ac_unknown_cflags_production=
lt_ac_unknown_cxxflags_debug=$lt_ac_unknown_cflags_debug
lt_ac_unknown_cxxflags_production=$lt_ac_unknown_cflags_production

dnl ----------------------------------------------------------------------------
dnl
dnl Expand to the repetition of a given string, $1 times.
dnl
dnl Input:
dnl   $1 = number of repetitions (integer, may also be 0 or negative)
dnl   $2 = string to repeat
dnl
dnl ----------------------------------------------------------------------------
m4_define([LT_AC_STR_REPEAT],[ifelse(m4_eval($1>0),0,,[$2[]LT_AC_STR_REPEAT(m4_eval($1-1),[$2])])])

dnl ----------------------------------------------------------------------------
dnl
dnl Expand to the given string, padded up to a certain length.
dnl
dnl Input:
dnl   $1 = length (should be a positive integer)
dnl   $2 = string
dnl   $3 = fill character
dnl
dnl ----------------------------------------------------------------------------
m4_define([LT_AC_STR_PAD],[$2[]LT_AC_STR_REPEAT(m4_eval($1-m4_len([$2])),[$3])])

dnl ----------------------------------------------------------------------------
dnl
dnl Write a message to "config.log".
dnl
dnl Input:
dnl   $1 = message
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_LOG],[cat >&AS_MESSAGE_LOG_FD << EOF
$1
EOF])

dnl ----------------------------------------------------------------------------
dnl
dnl Dump current C compiler settings to "config.log".
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_LOG_C_COMPILER_SETTINGS],[LT_AC_LOG([dnl
current C compiler settings:
  CC         = <$CC>
  CPPFLAGS   = <$CPPFLAGS>
  CFLAGS     = <$CFLAGS>
  ac_compile = <$ac_compile>])])

dnl ----------------------------------------------------------------------------
dnl
dnl Dump current C linker settings to "config.log".
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_LOG_C_LINKER_SETTINGS],[LT_AC_LOG([dnl
current C linker settings:
  CC       = <$CC>
  CFLAGS   = <$CFLAGS>
  LDFLAGS  = <$LDFLAGS>
  ac_link  = <$ac_link>])])

dnl ----------------------------------------------------------------------------
dnl
dnl Dump current C++ compiler settings to "config.log".
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_LOG_CXX_COMPILER_SETTINGS],[LT_AC_LOG([dnl
current C++ compiler settings:
  CXX        = <$CXX>
  CPPFLAGS   = <$CPPFLAGS>
  CXXFLAGS   = <$CXXFLAGS>
  ac_compile = <$ac_compile>])])

dnl ----------------------------------------------------------------------------
dnl
dnl Dump current C++ linker settings to "config.log".
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_LOG_CXX_LINKER_SETTINGS],[LT_AC_LOG([dnl
current C++ linker settings:
  CXX      = <$CXX>
  CXXFLAGS = <$CXXFLAGS>
  LDFLAGS  = <$LDFLAGS>
  ac_link  = <$ac_link>])])

dnl ----------------------------------------------------------------------------
dnl
dnl Print an error message complaining about a missing argument for a given
dnl option. Halt the script.
dnl
dnl Input:
dnl   $1 = relevant option
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MSG_ERROR_MISSING_ARG],[dnl
AC_MSG_ERROR([missing argument to "$1"; try "$[]0 --help".])])

dnl ----------------------------------------------------------------------------
dnl
dnl Print an error message complaining about a bad argument for a given option.
dnl Halt the script.
dnl
dnl Input:
dnl   $1 = relevant option
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MSG_ERROR_BAD_ARG],[dnl
AC_MSG_ERROR([bad argument to "$1"; try "$[]0 --help".])])

dnl ----------------------------------------------------------------------------
dnl
dnl Similar to AC_MSG_RESULT, but also provide an (optional) additional string.
dnl
dnl Input:
dnl   $1 = result string
dnl   $2 = additional string
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MSG_RESULT_USING],[dnl
if test x"$2" = x; then
  AC_MSG_RESULT([$1])
else
  AC_MSG_RESULT([$1 (using $2)])
fi])

dnl ----------------------------------------------------------------------------
dnl
dnl Output one or more lines of a message using AC_MSG_NOTICE.
dnl
dnl Input:
dnl   $1 = first line
dnl   .. = more lines
dnl
dnl ----------------------------------------------------------------------------
m4_define([LT_AC_MSG_NOTICE],[ifelse($#,0,,$#,1,[AC_MSG_NOTICE([$1])],[AC_MSG_NOTICE([$1])
LT_AC_MSG_NOTICE(m4_shift($@))])])

dnl ----------------------------------------------------------------------------
dnl
dnl Similar to AC_MSG_NOTICE, but specifically for hints; $lt_ac_has_hints is
dnl set to "yes" so to keep track of the message that was displayed. Later, when
dnl configure is finished, LT_AC_SUMMARY_FOOT will also print out a remininder
dnl for the user.
dnl
dnl Input:
dnl   $1               = message string (first line)
dnl   ..               = more lines
dnl
dnl Output:
dnl   $lt_ac_has_hints = "yes"
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MSG_HINT],[dnl
LT_AC_MSG_NOTICE($@)
lt_ac_has_hints=yes])

dnl ----------------------------------------------------------------------------
dnl
dnl Same as LT_AC_MSG_HINT, but for warnings.
dnl
dnl Input:
dnl   $1               = message string (first line)
dnl   ..               = more lines
dnl
dnl Output:
dnl   $lt_ac_has_warns = "yes"
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MSG_WARN],[dnl
LT_AC_MSG_NOTICE($@)
lt_ac_has_warns=yes])

dnl ----------------------------------------------------------------------------
dnl
dnl Expand the definition of a variable containing an installation directory.
dnl Don't use this macro before AC_OUTPUT, it may have adverse effects.
dnl
dnl Input:
dnl   $1 = variable name
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_EXPAND_DIR],[eval $1=`echo "$$1"`])

dnl ----------------------------------------------------------------------------
dnl
dnl Expand the definitions of all standard installation directories using
dnl LT_AC_EXPAND_DIR.
dnl Don't use this macro before AC_OUTPUT, it may have adverse effects.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_EXPAND_DIRS],[dnl
LT_AC_EXPAND_DIR(prefix)
LT_AC_EXPAND_DIR(exec_prefix)
LT_AC_EXPAND_DIR(bindir)
LT_AC_EXPAND_DIR(datadir)
LT_AC_EXPAND_DIR(includedir)
LT_AC_EXPAND_DIR(infodir)
LT_AC_EXPAND_DIR(libdir)
LT_AC_EXPAND_DIR(libexecdir)
LT_AC_EXPAND_DIR(localstatedir)
LT_AC_EXPAND_DIR(mandir)
LT_AC_EXPAND_DIR(oldincludedir)
LT_AC_EXPAND_DIR(sbindir)
LT_AC_EXPAND_DIR(sharedstatedir)
LT_AC_EXPAND_DIR(sysconfdir)])

dnl ----------------------------------------------------------------------------
dnl
dnl Rewrite a given variable, quoting the filename it contains, if needed (for
dnl example, if it contains spaces or other special characters).
dnl
dnl Please note that many characters other than those explicitely listed below
dnl in this macro *might* be accepted by the filesystem, but we choose to quote
dnl them anyway, both because their use is rare and - most importantly - because
dnl they're potentially not supported by all systems; after all, unnecessary
dnl quoting can do no harm.
dnl
dnl Input:
dnl   $1 = variable name
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_VAR_QUOTE],[dnl
if expr "$$1" : ".*[[^-./_$lt_digits$lt_locase_chars$lt_upcase_chars]]" > /dev/null; then
  $1="\"$$1\""
fi])

dnl ----------------------------------------------------------------------------
dnl
dnl Print a message to keep the user informed that we are taking into account
dnl the environment variable $1; this macro does nothing except notifying the
dnl user, so it should be put somewhere before the place where that variable is
dnl actually used.
dnl
dnl Input:
dnl   $1 = variable name
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_VAR_CHECKING],[dnl
AC_MSG_CHECKING(for environment variable $1)
if test x"$$1" = x; then
  AC_MSG_RESULT(not set)
else
  AC_MSG_RESULT([$$1])
fi])

dnl ----------------------------------------------------------------------------
dnl
dnl Define a preprocessor template.
dnl
dnl Input:
dnl   $1 = template name
dnl   $2 = description
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_TEMPLATE],[AH_TEMPLATE(AS_TR_CPP([$1]),[$2.])])

dnl ----------------------------------------------------------------------------
dnl
dnl Define a preprocessor template prefixed by "HAVE_".
dnl
dnl Input:
dnl   $1 = template name
dnl   $2 = description
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_TEMPLATE_HAVE],[LT_AC_TEMPLATE([HAVE_$1],[Define if you have $2])])

dnl ----------------------------------------------------------------------------
dnl
dnl Define a preprocessor template for a given header.
dnl
dnl Input:
dnl   $1 = header (with its ".h" suffix)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_TEMPLATE_HEADER],[LT_AC_TEMPLATE_HAVE([$1],[the <$1> header file])])

dnl ----------------------------------------------------------------------------
dnl
dnl Associate a value to a previously defined template.
dnl
dnl Input:
dnl   $1 = template name
dnl   $2 = value (may be null)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_DEFINE],[AC_DEFINE_UNQUOTED(AS_TR_CPP($1),[$2])])

dnl ----------------------------------------------------------------------------
dnl
dnl Confirm the definition of a template defined with LT_AC_TEMPLATE_HAVE, or
dnl LT_AC_TEMPLATE_HEADER.
dnl
dnl Input:
dnl   $1 = template name
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_HAVE],[LT_AC_DEFINE([HAVE_$1])])

dnl ----------------------------------------------------------------------------
dnl
dnl Patch PACKAGE_ definitions by prefixing them with LT_AH_PREFIX; needs $SED
dnl and $RM.
dnl
dnl Input:
dnl   $1 = filename
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_TRANSFORM_PACKAGE],[dnl
echo "patching $1"
$SED -e 's,LT_AH_PREFIX[]PACKAGE_,PACKAGE_,g' -e 's,PACKAGE_,LT_AH_PREFIX[]PACKAGE_,g' "$1" > "$1~" && $SED 's,x,x,' "$1~" > "$1"; $RM "$1~"])

dnl ----------------------------------------------------------------------------
dnl
dnl Initialize some library-related variables so that the package is actually
dnl disabled.
dnl
dnl Input:
dnl   $1           = package (library) name
dnl
dnl Output:
dnl   $header_$1   = cleared
dnl   $cppflags_$1 = cleared
dnl   $ldflags_$1  = cleared
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_INITIALIZE_WITH_LIBRARY],[dnl
header_$1=
cppflags_$1=
ldflags_$1=])

dnl ----------------------------------------------------------------------------
dnl
dnl Check $lt_have_lib and $lt_have_header to see what we should do about a
dnl library package.
dnl
dnl If both are set to "yes", we're happy, and we set $with_$1 to "yes" too.
dnl
dnl If either one is set to "no" and the other one is set to "yes", assume
dnl there's something wrong; so inform the user, and halt the script.
dnl
dnl Alternatively, if both are set to "no", we can get away if the library is
dnl not required (see below), but then we need to clear some library-related
dnl variables.
dnl
dnl Input:
dnl   $1              = library name
dnl   $lt_have_lib    = "yes"|"no"
dnl   $lt_have_header = "yes"|"no"
dnl   $require_$1     = "yes"|"no" (anything other than "no" means "yes")
dnl
dnl Output:
dnl   $with_$1        = "yes"|"no"
dnl   $header_$1      = see below (*)
dnl   $cppflags_$1    = see below (*)
dnl   $ldflags_$1     = see below (*)
dnl (*) cleared if $with_$1 is set to "no"; not touched otherwise.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_FINALIZE_WITH_LIBRARY],[dnl
case $lt_have_lib,$lt_have_header in
  yes,yes)
    with_$1=yes
    ;;
  yes,no)
    LT_AC_MSG_WARN([warning: you seem to have the library, but the relevant headers are],
                   [         missing; please check your installation.])
    LT_AC_MSG_HINT([hint:    the log file may give you additional details about what's],
                   [         actually wrong.])
    if test x"$require_$1" = xno; then
      LT_AC_MSG_HINT([hint:   since this library is optional, you may also want to disable],
                     [        support for it: try using "--without-$1".])
    fi
    AC_MSG_ERROR($1: headers not found.)
    ;;
  no,yes)
    LT_AC_MSG_WARN([warning: you don't have the library, but seem to have the headers],
                   [         around.])
    LT_AC_MSG_HINT([hint:    though this usually indicates a problem with the linker, the],
                   [         log file may give you more details about what's actually],
                   [         wrong with your installation.])
    if test x"$lt_ac_compiler" = xborland; then
      LT_AC_MSG_HINT([hint:    you are currently using Borland tools: if the problem is],
                     [         about linking against existing DLLs, make sure that you have],
                     [         all the import libraries (those silly ".lib" files)],
                     [         somewhere; if not, try using Borland's "implib" tool to],
                     [         create them.])
    fi
    if test x"$require_$1" = xno; then
      LT_AC_MSG_HINT([hint:   since this library is optional, you may also want to disable],
                     [        support for it: try using "--without-$1".])
    fi
    AC_MSG_ERROR([$1: library not found.])
    ;;
  *)
    case x$with_$1 in
      xauto|xno)
        if test x"$require_$1" = xno; then
          with_$1=no
          LT_AC_INITIALIZE_WITH_LIBRARY([$1])
        else
          AC_MSG_ERROR([$1 is a required library.])
        fi
        ;;
      *)
        AC_MSG_ERROR([$1: not found])
        ;;
    esac
    ;;
esac])

dnl ----------------------------------------------------------------------------
dnl
dnl Support stuff for list handling.
dnl
dnl ----------------------------------------------------------------------------
m4_define([LT_AC_LIST_CONCAT],[ifelse($#,0,,$#,1,[$1${lt_newline}],[$1${lt_newline}LT_AC_LIST_CONCAT(m4_shift($@))])])

dnl ----------------------------------------------------------------------------
dnl
dnl Clear a list.
dnl
dnl Input:
dnl   $1 = variable name
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_LIST_CLEAR],[$1=])

dnl ----------------------------------------------------------------------------
dnl
dnl Append one or more items to the end of a list. Please note that items are
dnl separated by newlines, with a trailing newline always ending the string
dnl (unless the list itself is empty).
dnl
dnl Input:
dnl   $1 = variable name
dnl   $2 = first item (if null, append an empty item)
dnl   .. = more items
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_LIST_APPEND],[$1="${$1}LT_AC_LIST_CONCAT(m4_shift($@))"])

dnl ----------------------------------------------------------------------------
dnl
dnl Loop over all items in a list. The caller may use "break" to quit the loop
dnl prematurely.
dnl
dnl Note that we can't just set $IFS to the newline character and do a "for"
dnl loop, since we want to be able to process empty items too; apparently bash,
dnl and possibly other shells too, does that only with other values of $IFS.
dnl
dnl Be extremely careful when using this macro. It's not so robust, and using it
dnl with non-list variables may produce infinite loops.
dnl
dnl This macro uses a temporary variable that is named after $1, so that nested
dnl usage is permitted, albeit on different lists.
dnl
dnl Input:
dnl   $1 = list variable name
dnl   $2 = item variable name
dnl   $3 = action to perform
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_LIST_WALK],[dnl
if test x"$$1" = x; then
  :
else
  lt_list_walk_$1=$$1
  while :
  do
    $2=${lt_list_walk_$1%%$lt_newline*}
    $3
    lt_list_walk_$1=${lt_list_walk_$1#*$lt_newline}
    test x"$lt_list_walk_$1" = x && break
  done
fi])

dnl ----------------------------------------------------------------------------
dnl
dnl Dump the contents of a list to "config.log".
dnl
dnl Input:
dnl   $1 = list variable name
dnl   $2 = optional comment
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_LIST_LOG],[dnl
if test x"$2" = x; then
  :
else
  LT_AC_LOG([$2:])
fi
LT_AC_LIST_WALK($1,lt_item,[LT_AC_LOG([  <$lt_item>])])])

dnl ----------------------------------------------------------------------------
dnl
dnl Determine the appropriate characters to use as path delimiter and separator;
dnl two corresponding preprocessor symbols are also defined.
dnl
dnl Output:
dnl   $lt_path_delim = path delimiter (*)
dnl   $lt_path_sep   = path separator (*)
dnl (*) AC_SUBST'ed.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_PATH_CHARS],[dnl

LT_AC_TEMPLATE(LT_AH_PREFIX[]PATH_DELIM,[Define to the one-character path delimiter for your platform; should be '/' for unix and '\\' for MS Windows])
LT_AC_TEMPLATE(LT_AH_PREFIX[]PATH_SEP,[Define to the one-character path separator for your platform; should be ':' for unix and ';' for MS Windows])

case $host in
  *-*-mingw32)
    lt_path_delim='\\'
    lt_path_sep=$PATH_SEPARATOR
    test x"$lt_path_sep" = x && lt_path_sep=';'
    ;;
  *)
    lt_path_delim='/'
    lt_path_sep=$PATH_SEPARATOR
    test x"$lt_path_sep" = x && lt_path_sep=':'
    ;;
esac

AC_SUBST(lt_path_delim)
AC_SUBST(lt_path_sep)

LT_AC_DEFINE(LT_AH_PREFIX[]PATH_DELIM,'$lt_path_delim')
LT_AC_DEFINE(LT_AH_PREFIX[]PATH_SEP,'$lt_path_sep')])

dnl ----------------------------------------------------------------------------
dnl
dnl If running on MinGW+MSYS, try to define $lt_windows_dir, $lt_system_dir and
dnl $lt_prgfiles_dir; this macro needs a working C environment, as failure in
dnl determining one of the three values causes the script to halt.
dnl
dnl If the host is not running on MinGW+MSYS, these three variables are cleared
dnl and no output is produced.
dnl
dnl Output:
dnl   $lt_windows_dir  = Windows directory (*1) (*2)
dnl   $lt_system_dir   = system directory (*1) (*2)
dnl   $lt_prgfiles_dir = program files directory (*1) (*2)
dnl (*1) with forward slashes.
dnl (*2) AC_SUBST'ed.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_GET_MSWIN_DIRS],[dnl
lt_windows_dir=
lt_system_dir=
lt_prgfiles_dir=
case $host in
  *-*-mingw32)
    AC_LANG_PUSH(C)
    AC_MSG_CHECKING(for MS Windows directory)
    AC_LINK_IFELSE([AC_LANG_PROGRAM([dnl
@%:@include <stdio.h>
@%:@include <windows.h>],[[
char buf[MAX_PATH];
if (GetWindowsDirectory(buf,sizeof buf)) puts(buf);]])],
                   [lt_windows_dir=`./conftest$ac_exeext`; lt_windows_dir=${lt_windows_dir//\\\\/\/}])
    if test x"$lt_windows_dir" = x; then
      AC_MSG_RESULT(error)
      AC_MSG_ERROR(cannot find MS Windows directory.)
    else
      AC_MSG_RESULT($lt_windows_dir)
    fi
    AC_MSG_CHECKING(for system directory)
    AC_LINK_IFELSE([AC_LANG_PROGRAM([dnl
@%:@include <stdio.h>
@%:@include <windows.h>],[[
char buf[MAX_PATH];
if (GetSystemDirectory(buf,sizeof buf)) puts(buf);]])],
                   [lt_system_dir=`./conftest$ac_exeext`; lt_system_dir=${lt_system_dir//\\\\/\/}])
    if test x"$lt_system_dir" = x; then
      AC_MSG_RESULT(error)
      AC_MSG_ERROR(cannot find system directory.)
    else
      AC_MSG_RESULT($lt_system_dir)
    fi
    AC_MSG_CHECKING(for program files directory)
    AC_LINK_IFELSE([AC_LANG_PROGRAM([dnl
@%:@include <stdio.h>
@%:@include <windows.h>],[[
HKEY hkey;
DWORD vtype;
char buf[MAX_PATH];
DWORD bufsize = sizeof buf;
if (RegOpenKey(HKEY_LOCAL_MACHINE,"Software\\\\Microsoft\\\\Windows\\\\CurrentVersion",&hkey) == ERROR_SUCCESS) {
  if (RegQueryValueEx(hkey,"ProgramFilesDir",0,&vtype,(BYTE*) &buf,&bufsize) == ERROR_SUCCESS && vtype == REG_SZ) puts(buf);
  RegCloseKey(hkey);
}]])],
                   [lt_prgfiles_dir=`./conftest$ac_exeext`; lt_prgfiles_dir=${lt_prgfiles_dir//\\\\/\/}])
    if test x"$lt_prgfiles_dir" = x; then
      AC_MSG_RESULT(error)
      AC_MSG_ERROR(cannot find program files directory.)
    else
      AC_MSG_RESULT($lt_prgfiles_dir)
    fi
    AC_LANG_POP
    ;;
esac
AC_SUBST(lt_windows_dir)
AC_SUBST(lt_system_dir)
AC_SUBST(lt_prgfiles_dir)])

dnl ----------------------------------------------------------------------------
dnl
dnl Search for a generic file, either in a specific directory or using the
dnl entire $PATH.
dnl
dnl The caller may examine each (directory) in which the file was found by
dnl looking at $lt_find_file_candidate, and optionally exit the loop by issuing
dnl a break.
dnl
dnl On MinGW+MSYS, the form "x:/path" for directory names listed in $PATH is
dnl preferred over "/x/path", as it makes easier to deal with other MS Windows
dnl tools.
dnl
dnl This macro has no output.
dnl
dnl Input:
dnl   $1 = filename
dnl   $2 = specific directory (if null, search in $PATH)
dnl   $3 = test option (if null, use "-f" by default)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_FIND_FILE],[dnl
LT_AC_LIST_CLEAR(lt_find_file_candidates)
if test x"$2" = x; then
  lt_find_file_save_ifs=$IFS
  IFS=$PATH_SEPARATOR
  for lt_find_file_candidate in $PATH
  do
    case $host in
      *-*-mingw32)
        lt_find_file_candidate=`echo "$lt_find_file_candidate" | sed 's,^/\([[A-Za-z]]\)/,\1:/,'`
        ;;
    esac
    LT_AC_LIST_APPEND(lt_find_file_candidates,$lt_find_file_candidate)
  done
  IFS=$lt_find_file_save_ifs
else
  LT_AC_LIST_APPEND(lt_find_file_candidates,[$2])
fi
LT_AC_LIST_WALK(lt_find_file_candidates,
                lt_find_file_candidate,
                [if test -d "$lt_find_file_candidate/$1"; then
                  :
                elif test ifelse([$3],,-f,[$3]) "$lt_find_file_candidate/$1"; then
                  $4
                fi])
])

dnl ----------------------------------------------------------------------------
dnl
dnl Wrap LT_AC_FIND_FILE, searching for an executable program and stopping at
dnl the first match. Unlike AC_CHECK_PROG and AC_PATH_PROG, this macro has no
dnl output.
dnl
dnl On return, $1 is empty if nothing was found.
dnl
dnl Input:
dnl   $1 = variable name
dnl   $2 = filename (without suffix, on MinGW+MSYS)
dnl   $3 = specific directory (if null, search in $PATH)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_FIND_PROG],[
$1=
LT_AC_FIND_FILE([$2],[$3],-x,[$1="$lt_find_file_candidate/$2"; break])
])

dnl ----------------------------------------------------------------------------
dnl
dnl Wrap AC_PATH_PROG, checking for a required program, and halt the script if
dnl nothing was found.
dnl
dnl Beware that this macro prints some output on its own.
dnl
dnl Input:
dnl   $1 = variable name
dnl   $2 = filename (without suffix, on MinGW+MSYS)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_REQUIRE_PROG],[dnl
$1=
AC_PATH_PROG([$1],[$2])
if test x"$$1" = x; then
  AC_MSG_ERROR([$2 is a required program.])
fi])

dnl ----------------------------------------------------------------------------
dnl
dnl Rewrite $ac_compile for C, so that we can insert some literal flags; this is
dnl particularly useful if these flags need some quoting.
dnl
dnl Input:
dnl   $1          = additional CPPFLAGS
dnl   $2          = additional CFLAGS
dnl
dnl Output:
dnl   $ac_compile = rewritten
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_REWRITE_AC_COMPILE_CC],[ac_compile="\$CC $1 \$CPPFLAGS $2 \$CFLAGS -c conftest.\$ac_ext >&AS_MESSAGE_LOG_FD"])

dnl ----------------------------------------------------------------------------
dnl
dnl Similar to LT_AC_REWRITE_AC_COMPILE_CC, but rewrite $ac_link instead.
dnl
dnl Input:
dnl   $1       = additional CFLAGS
dnl   $2       = additional LDFLAGS
dnl
dnl Output:
dnl   $ac_link = rewritten
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_REWRITE_AC_LINK_CC],[ac_link="\$CC $1 \$CFLAGS -o conftest$ac_exeext conftest.\$ac_ext $2 \$LDFLAGS >&AS_MESSAGE_LOG_FD"])

dnl ----------------------------------------------------------------------------
dnl
dnl Same as LT_AC_REWRITE_AC_COMPILE_C, but for C++.
dnl
dnl Input:
dnl   $1          = additional CPPFLAGS
dnl   $2          = additional CXXFLAGS
dnl
dnl Output:
dnl   $ac_compile = rewritten
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_REWRITE_AC_COMPILE_CXX],[ac_compile="\$CXX $1 \$CPPFLAGS $2 \$CXXFLAGS -c conftest.\$ac_ext >&AS_MESSAGE_LOG_FD"])

dnl ----------------------------------------------------------------------------
dnl
dnl Same as LT_AC_REWRITE_AC_LINK_C, but for C++.
dnl
dnl Input:
dnl   $1       = additional CXXFLAGS
dnl   $2       = additional LDFLAGS
dnl
dnl Output:
dnl   $ac_link = rewritten
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_REWRITE_AC_LINK_CXX],[ac_link="\$CXX $1 \$CXXFLAGS -o conftest$ac_exeext conftest.\$ac_ext $2 \$LDFLAGS >&AS_MESSAGE_LOG_FD"])

dnl ----------------------------------------------------------------------------
dnl
dnl Test for static libraries.
dnl
dnl Output:
dnl   $AEXT = suffix (*)
dnl   $TLIB = path to Borland's tlib (*)
dnl (*) AC_SUBST'ed.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_A_LIBRARIES],[dnl
AC_MSG_CHECKING(for suffix of static libraries)
lt_ac_use_tlib=no
if test x"$lt_ac_compiler" = xborland; then
  case $host in
    *-*-mingw32)
      lt_ac_use_tlib=yes
      ;;
  esac
fi
if test x"$lt_ac_use_tlib" = xyes; then
  AC_SUBST(AEXT,lib)
  LT_AC_MSG_RESULT_USING($AEXT,tlib)
  LT_AC_REQUIRE_PROG(TLIB,tlib)
else
  AC_SUBST(AEXT,a)
  LT_AC_MSG_RESULT_USING($AEXT,GNU ar)
  LT_AC_REQUIRE_PROG(AR,ar)
fi])

dnl ----------------------------------------------------------------------------
dnl
dnl Test for shared libraries.
dnl
dnl Output:
dnl   $SOEXT       = suffix (*)
dnl   $SO_CPPFLAGS = additional preprocessor flags (*)
dnl   $SO_CFLAGS   = additional compiler flags (*)
dnl   $SO_CXXFLAGS = additional compiler flags (*)
dnl   $SO_LDFLAGS  = additional linker flags (*)
dnl (*) AC_SUBST'ed.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_SO_LIBRARIES],[dnl
SO_CPPFLAGS=
SO_CFLAGS=
SO_CXXFLAGS=
SO_LDFLAGS=
LT_AC_TEMPLATE(LT_AH_PREFIX[]NEED_DECLSPEC,Define if you need the __declspec() stuff to build a DLL for MS Windows)
AH_BOTTOM([/* __declspec() stuff for MS Windows DLLs. */
@%:@ifdef ]LT_AH_PREFIX[NEED_DECLSPEC
  @%:@ifdef ]LT_AH_PREFIX[DLL
    @%:@define ]LT_AH_PREFIX[EXPORT __declspec(dllexport)
  @%:@else
    @%:@define ]LT_AH_PREFIX[EXPORT __declspec(dllimport)
  @%:@endif
@%:@else
  @%:@define ]LT_AH_PREFIX[EXPORT
@%:@endif])
AC_MSG_CHECKING(for suffix of shared libraries)
case $host in
  *-*-mingw32)
    AC_SUBST(SOEXT,dll)
    ;;
  *-*-darwin*)
    AC_SUBST(SOEXT,dylib)
    ;;
  *)
    AC_SUBST(SOEXT,so)
    ;;
esac
AC_MSG_RESULT($SOEXT)
AC_MSG_CHECKING(whether shared libraries need additional flags)
case $host in
  *-*-mingw32)
    if test x"$lt_ac_compiler" = xborland; then
      LT_AC_MSG_RESULT_USING(yes,-D[]LT_AH_PREFIX[]DLL)
      SO_CPPFLAGS="-D[]LT_AH_PREFIX[]DLL"
      LT_AC_DEFINE(LT_AH_PREFIX[]NEED_DECLSPEC)
    else
      LT_AC_MSG_RESULT_USING(yes,[-Wl,--export-all-symbols])
      SO_LDFLAGS="-Wl,--export-all-symbols"
    fi
    ;;
  *-*-darwin*)
    LT_AC_MSG_RESULT_USING(yes,-fno-common)
    SO_CFLAGS="-fno-common"
    SO_CXXFLAGS="-fno-common"
    ;;
  *)
    if test x"$lt_ac_compiler" = xborland; then
      AC_MSG_RESULT(no)
    else
      LT_AC_MSG_RESULT_USING(yes,-fPIC)
      SO_CFLAGS="-fPIC"
      SO_CXXFLAGS="-fPIC"
    fi
    ;;
esac
AC_SUBST(SO_CPPFLAGS)
AC_SUBST(SO_CFLAGS)
AC_SUBST(SO_CXXFLAGS)
AC_SUBST(SO_LDFLAGS)
case $host in
  *-*-mingw32)
    if test x"$lt_ac_compiler" = xborland; then
      LT_AC_REQUIRE_PROG(IMPLIB,implib)
      LT_AC_REQUIRE_PROG(IMPDEF,impdef)
    fi
    ;;
esac])

dnl ----------------------------------------------------------------------------
dnl
dnl Additional initialization code.
dnl
dnl ----------------------------------------------------------------------------
LT_AC_TEMPLATE([PACKAGE_VER_MAJOR],Major version)
LT_AC_DEFINE([PACKAGE_VER_MAJOR],[LT_AC_PKG_V_MAJOR])
LT_AC_TEMPLATE([PACKAGE_VER_MINOR],Minor version)
LT_AC_DEFINE([PACKAGE_VER_MINOR],[LT_AC_PKG_V_MINOR])
LT_AC_TEMPLATE([PACKAGE_VER_BUILD],Build number)
LT_AC_DEFINE([PACKAGE_VER_BUILD],[LT_AC_PKG_V_BUILD])
LT_AC_TEMPLATE([PACKAGE_TIMESTAMP],Define to the timestamp of this release)
LT_AC_DEFINE([PACKAGE_TIMESTAMP],["LT_AC_PKG_TIMESTAMP"])
AC_SUBST([PACKAGE_VER_MAJOR],["LT_AC_PKG_V_MAJOR"])
AC_SUBST([PACKAGE_VER_MINOR],["LT_AC_PKG_V_MINOR"])
AC_SUBST([PACKAGE_VER_BUILD],["LT_AC_PKG_V_BUILD"])
AC_SUBST([PACKAGE_TIMESTAMP],["LT_AC_PKG_TIMESTAMP"])
