################################################################################
#                                                                              #
# Copyright (C) 2003 Andrea Biardi                                             #
#                                                                              #
# This program is free software; you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation; either version 2 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
################################################################################

dnl ----------------------------------------------------------------------------
dnl
dnl Expand to a whitespace-separated list of filenames matching $1.
dnl
dnl Input:
dnl   $1 = filenames
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_LS],[m4_esyscmd(echo -n $1)])

dnl ----------------------------------------------------------------------------
dnl
dnl Expand to the directory portion of $1; if no directory portion is present in
dnl $1, expand to the default value given in $2.
dnl
dnl Beware that only forward slashes are considered.
dnl
dnl Input:
dnl   $1 = filename
dnl   $2 = default value (may be null)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_DIROF],[ifelse(m4_index($1,/),-1,$2,patsubst($1,/[[^/]]*$))])

dnl ----------------------------------------------------------------------------
dnl
dnl Expand to the filename portion of $1. Beware that only forward slashes are
dnl considered.
dnl
dnl Input:
dnl   $1 = filename
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_NODIR],[patsubst($1,^.*/)])

dnl ----------------------------------------------------------------------------
dnl
dnl Replace extension (suffix) of each whitespace-separated filename given; all
dnl filenames get their extension changed, even if they initially don't have
dnl one.
dnl
dnl Input:
dnl   $1 = filenames
dnl   $2 = new extension (*)
dnl (*) should have a leading dot; pass a null argument to remove the current
dnl     extension.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_CHEXT],[dnl
patsubst(AC_FOREACH([lt_file],
                    [$1],
                    [ ifelse(m4_index(lt_file,.),-1,lt_file[]$2,patsubst(lt_file,\.[[^.]]*$,[$2]))]),
         [^ ])])

dnl ----------------------------------------------------------------------------
dnl
dnl Fill a given variable with the flags needed to link against a static
dnl library; the file itself needs not exist (yet).
dnl
dnl Warning: use this macro only if you are sure that filename quoting through
dnl LT_AC_VAR_QUOTE is not necessary.
dnl
dnl Input:
dnl   $1  = variable name
dnl   $2  = library path and filename
dnl
dnl Output:
dnl   $$1 = linker flags
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_LDFLAGS_STATIC],[$1="$2"])

dnl ----------------------------------------------------------------------------
dnl
dnl Fill a given variable with the flags needed to link against a shared
dnl library.
dnl
dnl Warning: use this macro only if you are sure that filename quoting through
dnl LT_AC_VAR_QUOTE is not necessary.
dnl
dnl Input:
dnl   $1  = variable name
dnl   $2  = library path and filename
dnl
dnl Output:
dnl   $$1 = linker flags
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_LDFLAGS_SHARED],[dnl
$1=
case $host in
  *-*-mingw32)
    if test x"$lt_ac_compiler" = xborland; then
      $1="LT_AC_CHEXT($2,.lib)"
    else
      $1="$2"
    fi
    ;;
  *)
    $1="-L[]LT_AC_DIROF($2) -l[]patsubst(LT_AC_NODIR(LT_AC_CHEXT($2)),^lib)"
    ;;
esac])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit an empty rule.
dnl
dnl Input:
dnl   $1 = targets
dnl   $2 = prerequisites
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MK_RULE],[dnl
echo
echo "$1: $2"])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit an empty phony rule.
dnl
dnl Input:
dnl   $1 = targets
dnl   $2 = prerequisites
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MK_PHONY],[dnl
echo
echo ".PHONY: $1"
echo "$1: $2"])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit a make rule describing how to create a dependency file out of a C++
dnl source.
dnl
dnl Note: this rule is skipped if $CXXCPP_M is empty.
dnl
dnl This rule needs @CXXCPP_M@ and @SED@.
dnl
dnl Input:
dnl   $1 = source (C++ file)
dnl   $2 = additional prerequisites
dnl   $3 = additional preprocessor flags
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MK_D_CXX_1],[dnl
if test x"$CXXCPP_M" = x; then
  :
else
  echo
  echo "-include LT_AC_CHEXT($1,.d)"
  echo
  echo "LT_AC_CHEXT($1,.d): $2"
  if test x"$CXXCPP_M_OUT" = xstderr; then
    echo "	@CXXCPP_M@ $3 \$(CPPFLAGS) $1 2>&1 1>/dev/null | @SED@ 's,^LT_AC_CHEXT(LT_AC_NODIR($1),\\.@OBJEXT@)[[	 ]]*:,LT_AC_CHEXT($1,.d) LT_AC_CHEXT($1,.@OBJEXT@):,' > LT_AC_CHEXT($1,.d)"
  else
    echo "	@CXXCPP_M@ $3 \$(CPPFLAGS) $1 | @SED@ 's,^LT_AC_CHEXT(LT_AC_NODIR($1),\\.@OBJEXT@)[[	 ]]*:,LT_AC_CHEXT($1,.d) LT_AC_CHEXT($1,.@OBJEXT@):,' > LT_AC_CHEXT($1,.d)"
  fi
fi])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit one rule for each filename in $1, using LT_AC_MK_D_CXX_1.
dnl
dnl Input:
dnl   $1 = sources (multiple C++ files)
dnl   $2 = additional prerequisites (for each target)
dnl   $3 = additional preprocessor flags
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MK_D_CXX],[dnl
AC_FOREACH([lt_file],
           [$1],
           [LT_AC_MK_D_CXX_1([lt_file],[$2],[$3])]
)])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit a make rule describing how to create an object file out of a C++
dnl source.
dnl
dnl This rule needs @CXX@.
dnl
dnl Input:
dnl   $1 = target (object file)
dnl   $2 = source (C++ file)
dnl   $3 = additional prerequisites
dnl   $4 = additional preprocessor flags
dnl   $5 = additional compiler flags
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MK_O_CXX_1],[dnl
echo
echo "$1: $2 $3"
echo "	@CXX@ $4 \$(CPPFLAGS) $5 \$(CXXFLAGS) ${lt_compiler_o_option}$1 -c $2"])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit one rule for each filename in $1, using LT_AC_MK_O_CXX_1.
dnl
dnl Input:
dnl   $1 = sources (multiple C++ files)
dnl   $2 = additional prerequisites (for each target)
dnl   $3 = additional preprocessor flags
dnl   $4 = additional compiler flags
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MK_O_CXX],[dnl
AC_FOREACH([lt_file],
           [$1],
           [LT_AC_MK_O_CXX_1([LT_AC_CHEXT(lt_file,.@OBJEXT@)],[lt_file],[$2],[$3],[$4])]
)])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit a make rule describing how to create an archive of object files, also
dnl known as a static library.
dnl
dnl This rule may need @RM@, @TLIB@ and @AR@.
dnl
dnl A couple of notes for tlib: first, this one doesn't like forward slashes,
dnl and then we must also make sure the target doesn't exist beforehand, or tlib
dnl will complain about (possibly outdated) object files already being there and
dnl refuse to replace them.
dnl
dnl Input:
dnl   $1 = target (archive of object files)
dnl   $2 = sources (multiple object files)
dnl   $3 = additional prerequisites
dnl   $4 = page size for tlib (default = 16)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MK_A],[dnl
echo
echo "$1: $2 $3"
lt_ac_use_tlib=no
if test x"$lt_ac_compiler" = xborland; then
  case $host in
    *-*-mingw32)
      lt_ac_use_tlib=yes
      ;;
  esac
fi
if test x"$lt_ac_use_tlib" = xyes; then
  echo "	@RM@ -f $1"
  echo "	@TLIB@ //p[]ifelse([$4],,16,[$4]) patsubst($1,/,\\\\\\) //a patsubst($2,/,\\\\\\)"
else
  echo "	@AR@ rcs $1 $2"
fi])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit a make rule describing how to create a shared library out of one or
dnl more C++ object files.
dnl
dnl Please note that $6, $7 and $8 must be evaluated at call time; also, they
dnl must not be all zero.
dnl
dnl This rule needs @CXX@, and may need @IMPLIB@ and @IMPDEF@.
dnl
dnl Input:
dnl   $1 = target (shared library)
dnl   $2 = sources (multiple object files)
dnl   $3 = additional prerequisites
dnl   $4 = additional compiler flags
dnl   $5 = additional linker flags
dnl   $6 = library name
dnl   $7 = major version
dnl   $8 = minor version
dnl   $9 = build number
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MK_SO_CXX],[dnl
echo
echo "$1: $2 $3"
case $host in
  *-*-mingw32)
    if test x"$lt_ac_compiler" = xborland; then
      echo "	@CXX@ -tWD $4 \$(CXXFLAGS) ${lt_linker_o_option}$1 $2 $5 \$(LDFLAGS)"
      echo "	@IMPLIB@ LT_AC_CHEXT($1,.lib) $1"
      echo "	@IMPDEF@ LT_AC_CHEXT($1,.def) $1"
    else
      echo "	@CXX@ -shared $4 \$(CXXFLAGS) ${lt_linker_o_option}$1 $2 -Wl,--out-implib,LT_AC_CHEXT($1,.lib) -Wl,--output-def,LT_AC_CHEXT($1,.def) $5 \$(LDFLAGS)"
    fi
    ;;
  *-*-darwin*)
    if test $7 -eq 0 && test $8 -eq 0; then
      echo "	@CXX@ -dynamiclib -install_name $6.$7.@SOEXT@ -current_version $7.$8.$9 $4 \$(CXXFLAGS) ${lt_linker_o_option}$1 $2 $5 \$(LDFLAGS)"
    else
      echo "	@CXX@ -dynamiclib -install_name $6.$7.@SOEXT@ -compatibility_version $7.$8 -current_version $7.$8.$9 $4 \$(CXXFLAGS) ${lt_linker_o_option}$1 $2 $5 \$(LDFLAGS)"
    fi
    ;;
  *)
    if test x"$lt_ac_compiler" = xborland; then
      echo "	@CXX@ -tD $4 \$(CXXFLAGS) ${lt_linker_o_option}$1 $2 -lN$6.@SOEXT@.$7 $5 \$(LDFLAGS)"
    else
      echo "	@CXX@ -shared $4 \$(CXXFLAGS) ${lt_linker_o_option}$1 $2 -Wl,-soname,$6.@SOEXT@.$7 $5 \$(LDFLAGS)"
    fi
    ;;
esac])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit a make rule describing how to create an executable out of one or more
dnl C++ object files.
dnl
dnl This rule needs @CXX@, and makes optional use of @STRIP@.
dnl
dnl Input:
dnl   $1 = target (executable file)
dnl   $2 = sources (multiple object files)
dnl   $3 = additional prerequisites
dnl   $4 = additional compiler flags
dnl   $5 = additional linker flags
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MK_E_CXX_1],[dnl
echo
echo "$1: $2 $3"
echo "	@CXX@ $4 \$(CXXFLAGS) ${lt_linker_o_option}$1 $2 $5 \$(LDFLAGS)"
test x"$STRIP" = x || echo "	@STRIP@ $1"])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit one rule for each filename in $1, using LT_AC_MK_E_CXX_1.
dnl
dnl Input:
dnl   $1 = sources (multiple object files)
dnl   $2 = additional prerequisites (for each target)
dnl   $3 = additional compiler flags
dnl   $4 = additional linker flags
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MK_E_CXX],[dnl
AC_FOREACH([lt_file],
           [$1],
           [LT_AC_MK_E_CXX_1([LT_AC_CHEXT(lt_file,@EXEEXT@)],[lt_file],[$2],[$3],[$4])]
)])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit a make rule describing how to create a DVI file out of a TeX source; we
dnl need two passes to get the table of contents correct.
dnl
dnl This rule needs @LATEX@.
dnl
dnl Input:
dnl   $1 = source (LaTeX file)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MK_DVI_TEX],[dnl
echo
echo "LT_AC_CHEXT($1,.dvi): $1"
echo "	cd LT_AC_DIROF($1,.) && @LATEX@ -interaction=nonstopmode LT_AC_NODIR($1)"
echo "	cd LT_AC_DIROF($1,.) && @LATEX@ -interaction=nonstopmode LT_AC_NODIR($1)"])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit a make rule describing how to create a PostScript file out of a DVI
dnl source.
dnl
dnl This rule needs @DVIPS@.
dnl
dnl Input:
dnl   $1 = target (PostScript file)
dnl   $2 = source (DVI file)
dnl   $3 = paper size ("a4", "letter", etc.)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MK_PS_DVI],[dnl
echo
echo "$1: $2"
echo "	@DVIPS@ -t $3 -o $1 $2"])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit a make rule describing how to create a PDF file out of a DVI source.
dnl
dnl This rule needs @DVIPDFM@.
dnl
dnl Input:
dnl   $1 = target (PDF file)
dnl   $2 = source (DVI file)
dnl   $3 = paper size ("a4", "letter", etc.)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MK_PDF_DVI],[dnl
echo
echo "$1: $2"
echo "	@DVIPDFM@ -p $3 -o $1 $2"])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit make commands to remove a list of files.
dnl
dnl This macro needs @RM@.
dnl
dnl Input:
dnl   $1 = files to remove (wildcards ok, separate with spaces)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MC_CLEAN],[dnl
cat << EOF
	@RM@ -f $1
EOF])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit make commands to remove all files and directories but those explicitely
dnl listed. This thing is potentially dangerous: use with care!
dnl
dnl This macro needs @FIND@, @XARGS@ and @RM@.
dnl
dnl Input:
dnl   $1 = directory (must not be empty)
dnl   $2 = files/directories to preserve (wildcards ok, separate with spaces)
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MC_CLEAN_BUT],[dnl
cat << EOF
	@@FIND@ $1 -maxdepth 1 ! -name 'LT_AC_NODIR([$1])'[]AC_FOREACH([lt_file],[$2],[ ! -name 'lt_file']) | @XARGS@ -r -t @RM@ -r -f
EOF])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit make commands to create an installation directory.
dnl
dnl This macro needs @INSTALL@.
dnl
dnl Input:
dnl   $1 = installation directory
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MC_INSTDIR],[dnl
cat << EOF
	@INSTALL@ -d "$1"
EOF])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit make commands to install one or more files in an existing directory.
dnl
dnl This macro needs @INSTALL@.
dnl
dnl Input:
dnl   $1 = source files (separate with spaces)
dnl   $2 = destination directory
dnl   $3 = install options (default is "-m 644")
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MC_INSTALL],[dnl
cat << EOF
AC_FOREACH([lt_file],[$1],[	@INSTALL@ ifelse([$3],,-m 644,[$3]) lt_file "$2"
])EOF])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit make commands to install a shared library in an existing directory.
dnl
dnl This macro needs @INSTALL@, and may need @LN_S@.
dnl
dnl Input:
dnl   $1 = source file
dnl   $2 = destination directory
dnl   $3 = library name
dnl   $4 = major version
dnl   $5 = minor version
dnl   $6 = build number
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_MC_INSTALL_SO],[dnl
case $host in
  *-*-mingw32)
    echo "	@INSTALL@ -m 644 $1 \"$2\""
    echo "	@INSTALL@ -m 644 LT_AC_CHEXT($1,.lib) \"$2\""
    echo "	@INSTALL@ -m 644 LT_AC_CHEXT($1,.def) \"$2\""
    ;;
  *-*-darwin*)
    echo "	@INSTALL@ -m 755 $1 \"$2/$3.$4.$5.$6.@SOEXT@\""
    echo "	cd \"$2\" && @LN_S@ -f $3.$4.$5.$6.@SOEXT@ $3.$4.@SOEXT@"
    echo "	cd \"$2\" && @LN_S@ -f $3.$4.@SOEXT@ $3.@SOEXT@"
    ;;
  *)
    echo "	@INSTALL@ -m 755 $1 \"$2/$3.@SOEXT@.$4.$5.$6\""
    echo "	cd \"$2\" && @LN_S@ -f $3.@SOEXT@.$4.$5.$6 $3.@SOEXT@.$4"
    echo "	cd \"$2\" && @LN_S@ -f $3.@SOEXT@.$4 $3.@SOEXT@"
    ;;
esac])
