################################################################################
#                                                                              #
# Copyright (C) 2005 Andrea Biardi                                             #
#                                                                              #
# This program is free software; you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation; either version 2 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
################################################################################

dnl ----------------------------------------------------------------------------
dnl
dnl Provide a "--enable-parallel-make" option to enable parallel job execution;
dnl please note that this option supports GNU make only, and not all compilers
dnl (notably, not the ones from Borland).
dnl
dnl Input:
dnl   $lt_ac_compiler       = "gnu"|"borland"|"intel"|"unknown"
dnl
dnl Output:
dnl   $enable_parallel_make = "yes"|"no" (*)
dnl (*) AC_SUBST'ed.
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_ENABLE_PARALLEL_MAKE],[dnl

AC_ARG_ENABLE(parallel-make,
              AC_HELP_STRING([--enable-parallel-make],
                             [enable parallel make job execution (GNU make only)] [[[no]]]),
              [],
              enable_parallel_make=no)

AC_MSG_CHECKING(whether to enable parallel make job execution)

case x$enable_parallel_make in
  x)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_MISSING_ARG(--enable-parallel-make)
    ;;
  xyes)
    case x$lt_ac_compiler in
      xgnu|xintel)
        AC_MSG_RESULT(yes)
        ;;
      xborland)
        AC_MSG_RESULT(error)
        AC_MSG_ERROR(Borland C/C++ compilers do not support "--enable-parallel-make")
        ;;
      *)
        AC_MSG_RESULT(yes)
        LT_AC_MSG_WARN([warning: the compiler you are using has not been checked against parallel make job],
                       [         execution; if the build fails mysteriously, try "--disable-parallel-make"],
                       [         before reporting problems.])
        ;;
    esac
    ;;
  xno)
    AC_MSG_RESULT(no)
    ;;
  *)
    AC_MSG_RESULT(error)
    LT_AC_MSG_ERROR_BAD_ARG(--enable-parallel-make)
    ;;
esac

AC_SUBST(enable_parallel_make)

])

dnl ----------------------------------------------------------------------------
dnl
dnl Emit a makefile line containing make flags for (optional) parallel make job
dnl execution (GNU make only); the line will be commented out unless
dnl $enable_parallel_make is set to "yes".
dnl
dnl The makefile, of course, must itself support this option.
dnl
dnl Note: we are not using "MAKEFLAGS += j" because GNU make would not set the
dnl variable unless this is the toplevel makefile. Go figure out why this one
dnl works instead.
dnl
dnl Also, the maximum number of concurrent processes is predefined to 4.
dnl
dnl Input:
dnl   $enable_parallel_make = "yes"|"no" (anything other than "yes" means "no")
dnl
dnl ----------------------------------------------------------------------------
AC_DEFUN([LT_AC_PARALLEL_MAKE_FLAGS],[dnl
echo
echo "# this Makefile supports parallel job execution; if you are using GNU make, you"
echo "# may want to say "make -j" or uncomment the following line to take advantage of"
echo "# this feature"
if test x"$enable_parallel_make" = xyes; then
  echo 'MAKEFLAGS := $(MAKEFLAGS)j4'
else
  echo '#MAKEFLAGS := $(MAKEFLAGS)j4'
fi])
