/*******************************************************************************

Copyright (C) 2003 Andrea Biardi - LOGICOMM

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#include <iostream>
#include <libinterbasexx/libinterbasexx.h>

int main() {

  int result = 0;

  try {

    // attach to database

    IB::Database db("localhost:/opt/interbase/examples/employee.gdb","SYSDBA","masterkey");

    std::cout << std::endl;
    std::cout << "attaching to " << db.database() << " as " << db.username() << std::endl;

    db.attach();

    // start a transaction

    IB::Transaction tr(db);
    tr.start();

    // perform a query

    IB::Query q(db,tr,"select job_title, max_salary from job where max_salary >= ?");

    q.in()[0] = 100000.0;
    q.open();

    std::cout << std::endl << "showing jobs with a high salary" << std::endl << std::endl;
    while (q.fetch()) {
      for (int i = 0; i < q.out().count(); ++i) std::cout << q.out()[i] << '\t';
      std::cout << std::endl;
    }

    // close it, change parameters, and open the same query again

    q.close();

    q.in()[0] = 150000.0;
    q.open();

    std::cout << std::endl << "showing jobs with a very high salary" << std::endl << std::endl;
    while (q.fetch()) {
      for (int i = 0; i < q.out().count(); ++i) std::cout << q.out()[i] << '\t';
      std::cout << std::endl;
    }

    // no need to end the transaction or detach from database, since that's done
    // automatically

  }
  catch (const IB::Exception& e) {
    std::cerr << e.msg() << std::endl;
    result = 1;
  }

  return result;

}
