/*******************************************************************************

Copyright (C) 2003 Andrea Biardi - LOGICOMM

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#include <libinterbasexx/array.h>

namespace IB {

std::ostream& Array::put(std::ostream& stream) const {
  return stream << "<array:" << fid.isc_quad_high << ':' << fid.isc_quad_low << '>';
}

ArrayDesc::ArrayDesc(const Database& database, const Transaction& transaction, const std::string& table, const std::string& column) {

  // initialize things

  ISC_STATUS v[20];
  isc_db_handle dh = database.handle();
  isc_db_handle th = transaction.handle();

  // retrieve information for this column (fills fdesc)

  if (isc_array_lookup_bounds(v,&dh,&th,(char*) table.c_str(),(char*) column.c_str(),&fdesc)) throw EInterbase(v);

  // calculate (and normalize) array dimensions

  fsize = fdesc.array_desc_dimensions < 0 ? 0 : 1;
  for (short i = 0; i < fdesc.array_desc_dimensions; ++i) fsize *= 1+fdesc.array_desc_bounds[i].array_bound_upper-fdesc.array_desc_bounds[i].array_bound_lower;

}

ISC_LONG readArrayBuffer(const Database& database, const Transaction& transaction, const ArrayDesc& arrayDesc, const Array& array, void* data, ISC_LONG size) {

  // initialize things

  ISC_STATUS v[20];
  isc_db_handle dh = database.handle();
  isc_db_handle th = transaction.handle();

  // check the size of the buffer

  ISC_LONG result = arrayBufferSize(arrayDesc);
  if (result > size) throw EInterbase(format("%s: Buffer too small for this array (allocated = %d, needed = %d)",arrayDesc.column(),size,result));

  // read in the actual content of the array

  ISC_QUAD id = array.id();
  if (isc_array_get_slice(v,&dh,&th,&id,(ISC_ARRAY_DESC*) arrayDesc.desc(),data,&result)) throw EInterbase(v);

  // return value is the number of bytes filled in

  IBXX_ASSERT(result == arrayBufferSize(arrayDesc));
  return result;

}

ISC_LONG readArrayBuffer(const Database& database, const Transaction& transaction, const ArrayDesc& arrayDesc, const Array& array, CBuffer& buffer) {

  // resize the buffer, if needed

  ISC_LONG z = arrayBufferSize(arrayDesc);
  if (buffer.size() < z) buffer.resize(IBXX_ALIGN_SIZE(z));

  return readArrayBuffer(database,transaction,arrayDesc,array,buffer.data(),buffer.size());

}

Array writeArrayBuffer(const Database& database, const Transaction& transaction, const ArrayDesc& arrayDesc, const void* data, ISC_LONG size) {

  // initialize things

  ISC_STATUS v[20];
  isc_db_handle dh = database.handle();
  isc_db_handle th = transaction.handle();

  // write out the actual content of the array
  //
  // unlike writeBlob(), writeArrayBuffer() returns a *temporary* ID, suitable
  // for "insert" and "update" SQL statements
  //
  // once the array is inserted or updated, the ID changes (even if the
  // transaction has not been committed yet); the only way to retrieve the real
  // (final) ID, if needed, is to re-"select" the column

  ISC_QUAD id;
  std::memset(&id,0,sizeof id);

  if (isc_array_put_slice(v,&dh,&th,&id,(ISC_ARRAY_DESC*) arrayDesc.desc(),(void*) data,&size)) throw EInterbase(v);

  // return the new array ID

  return Array(id);

}

} // namespace IB
