/*******************************************************************************

Copyright (C) 2003 Andrea Biardi - LOGICOMM

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#ifndef __IBXX_BLOB_H__
#define __IBXX_BLOB_H__

/*******************************************************************************

Blob class and functions to read/write blob columns.

*******************************************************************************/

#include <cstring>
#include <iostream>
#include <libinterbasexx/config.h>
#include <libinterbasexx/cxxbits/cbuffer.h>
#include <libinterbasexx/database.h>
#include <libinterbasexx/transaction.h>
#include <string>
#include <ibase.h>

namespace IB {

class IBXX_EXPORT Blob {
  private:
    ISC_QUAD fid; // blob ID
  public:
    Blob() { std::memset(&fid,0,sizeof fid); }
    Blob(const ISC_QUAD& id): fid(id) {}
    const ISC_QUAD& id() const { return fid; }
    std::ostream& put(std::ostream& stream) const;
};

inline std::ostream& operator<<(std::ostream& stream, const Blob& v) {
  return v.put(stream);
}

ISC_LONG IBXX_EXPORT readBlob(const Database& database, const Transaction& transaction, const Blob& blob, void* data, ISC_LONG maxsize);
ISC_LONG IBXX_EXPORT readBlob(const Database& database, const Transaction& transaction, const Blob& blob, CBuffer& buffer, ISC_LONG maxsize = 0);
std::string IBXX_EXPORT readBlob(const Database& database, const Transaction& transaction, const Blob& blob, ISC_LONG maxsize = 0);

inline ISC_LONG blobSize(const Database& database, const Transaction& transaction, const Blob& blob, ISC_LONG maxsize = 0) {
  return readBlob(database,transaction,blob,0,maxsize);
}

Blob IBXX_EXPORT writeBlob(const Database& database, const Transaction& transaction, const void* data, ISC_LONG size);

inline Blob writeBlob(const Database& database, const Transaction& transaction, const CBuffer& buffer) {
  return writeBlob(database,transaction,buffer.data(),buffer.size());
}

inline Blob writeBlob(const Database& database, const Transaction& transaction, const std::string& buffer) {
  return writeBlob(database,transaction,buffer.c_str(),buffer.size());
}

} // namespace IB

#endif // __IBXX_BLOB_H__
