/*******************************************************************************

Copyright (C) 2003 Andrea Biardi - LOGICOMM

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#include <cstring>
#include <libinterbasexx/cxxbits/cbuffer.h>
#include <libinterbasexx/cxxbits/exception.h>
#include <libinterbasexx/cxxbits/format.h>

namespace IB {

CBuffer& CBuffer::copy(const CBuffer& src) {
  resize(src.size());
  std::memcpy(fdata,src.data(),fsize);
  return *this;
}

void CBuffer::resize(int value) {

  // we don't want to use realloc() if value == 0, since we cannot catch when
  // block is actually freed (realloc() might return a non-null pointer even
  // when the new size is 0); on the contrary, it's always safe to call free(),
  // even if fdata == 0 (just useless)

  if (value == fsize) return;

  if (value) {
    void* p;
    if ((p = std::realloc(fdata,value)) == 0) throw ESystem(format("call to realloc() failed (old size = %d, new size = %d)",fsize,value));
    fsize = value;
    fdata = p;
  } else {
    std::free(fdata);
    fsize = 0;
    fdata = 0;
  }

}

} // namespace IB
