/*******************************************************************************

Copyright (C) 2003 Andrea Biardi - LOGICOMM

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#ifndef __IBXX_CXXBITS_CBUFFER_H__
#define __IBXX_CXXBITS_CBUFFER_H__

/*******************************************************************************

Basic class for resizable buffers.

*******************************************************************************/

#include <cstdlib>
#include <libinterbasexx/cxxbits/config.h>

namespace IB {

class IBXX_EXPORT CBuffer {
  private:
    int fsize;   // allocated size (may be 0)
    void* fdata; // pointer to data (or 0, when fsize == 0)
  protected:
    CBuffer& copy(const CBuffer& src);
  public:
    CBuffer(): fsize(0), fdata(0) {}
    CBuffer(int size): fsize(size) { fdata = fsize ? std::malloc(fsize) : 0; }
    CBuffer(const CBuffer& src): fsize(0), fdata(0) { copy(src); }
    ~CBuffer() { std::free(fdata); }
    int size() const { return fsize; }
    const void* data() const { return fdata; }
    void* data() { return fdata; }
    void resize(int value);
    CBuffer& operator=(const CBuffer& value) { return copy(value); }
};

} // namespace IB

#endif // __IBXX_CXXBITS_CBUFFER_H__
