/*******************************************************************************

Copyright (C) 2004 Andrea Biardi

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#ifndef __IBXX_CXXBITS_EXCEPTION_H__
#define __IBXX_CXXBITS_EXCEPTION_H__

/*******************************************************************************

Base exception class and commonly-used exceptions.

*******************************************************************************/

#include <libinterbasexx/cxxbits/config.h>
#include <string>

namespace IB {

class IBXX_EXPORT Exception {
  private:
    std::string fmsg; // error message
  protected:
    void setMsg(const std::string& value) { fmsg = value; }
  public:
    Exception() {}
    Exception(const std::string& msg): fmsg(msg) {}
    virtual ~Exception() {}
    const std::string& msg() const { return fmsg; }
};

class IBXX_EXPORT ESystem: public Exception {
  private:
    int fno; // system error number
  public:
    ESystem(int no = 0);
    ESystem(const std::string& msg, int no = 0);
    int no() const { return fno; }
};

class IBXX_EXPORT EConv: public Exception {
  private:
    std::string fstr;           // string that failed the conversion
    std::string::size_type fat; // offset at which the conversion failed
  public:
    EConv(const std::string& msg, const std::string& str, std::string::size_type at = 0): Exception(msg), fstr(str), fat(at) {}
    const std::string& str() const { return fstr; }
    std::string::size_type at() const { return fat; }
};

class IBXX_EXPORT EMath: public Exception {
  public:
    EMath(const std::string& msg = ""): Exception(msg) {}
};

} // namespace IB

#endif // __IBXX_CXXBITS_EXCEPTION_H__
