/*******************************************************************************

Copyright (C) 2004 Andrea Biardi

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#include <libinterbasexx/cxxbits/format.h>
#include <libinterbasexx/cxxbits/strmisc.h>

namespace IB {

std::string& lcase(std::string& s) {
  for (std::string::iterator p = s.begin(); p != s.end(); ++p) *p = lcase_c(*p);
  return s;
}

std::string& ucase(std::string& s) {
  for (std::string::iterator p = s.begin(); p != s.end(); ++p) *p = ucase_c(*p);
  return s;
}

std::string& cap(std::string& s, const std::string& t) {
  char c = 0;
  for (std::string::iterator p = s.begin(); p != s.end(); c = *p++) *p = (p == s.begin() || t.find(c) != std::string::npos) ? ucase_c(*p) : lcase_c(*p);
  return s;
}

std::string& strreplace(std::string& s, const std::string& t, const std::string& u) {

  // given the string s, replace all occurrences of t with u; replacement is not
  // recursive

  std::string::size_type i = 0;

  while ((i = s.find(t,i)) != std::string::npos) {
    s.replace(i,t.length(),u);
    i += u.length();
  }

  return s;

}

std::string& strquote(std::string& s, char c) {

  // quote the string s using the given character; any preexisting occurrence of
  // c inside the string is doubled

  std::string::size_type i = 0;

  while ((i = s.find(c,i)) != std::string::npos) {
    s.insert(++i,1,c);
    ++i;
  }

  return s.insert((std::string::size_type)0,1,c).append(1,c);

}

int str2int(const char* s) {
  int result;
  if (!a2i(s,result)) throw EConv(format("\"%s\" is not a valid integer value",s),s);
  return result;
}

int str2intdef(const char* s, int def) {
  int result;
  return a2i(s,result) ? result : def;
}

std::string int2str(int value, int precision, int base, const char* digits, char psign) {
  return i2a(value,precision,base,digits,psign);
}

std::string concat(const std::string& s, int n, const std::string& t) {
  std::string result;
  result.reserve(n*s.length()+(n-1)*t.length());
  while (n-- > 0) {
    result += s;
    if (n) result += t;
  }
  return result;
}

} // namespace IB
