/*******************************************************************************

Copyright (C) 2005 Andrea Biardi

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#include <cstring>
#include <libinterbasexx/cxxbits/exception.h>
#include <libinterbasexx/cxxbits/timestamp.h>
#include <errno.h>

namespace IB {

Timestamp::Timestamp(int year, int month, int day, int hour, int min, int sec) {

  std::tm t;
  std::memset(&t,0,sizeof t);

  t.tm_year = year-1900;
  t.tm_mon = --month;
  t.tm_mday = day;
  t.tm_hour = hour;
  t.tm_min = min;
  t.tm_sec = sec;

  copyFrom(t);

}

Timestamp& Timestamp::copyFrom(std::tm value) {

  std::time_t t;
  if ((t = std::mktime(&value)) == (std::time_t)-1) throw ESystem(EINVAL);

  ftime = t;
  return *this;

}

std::tm& Timestamp::copyTo(std::tm& value) const {

  std::time_t t = ftime;
  std::tm* m;

  if ((m = std::localtime(&t)) == 0) throw ESystem(EINVAL);
  return value = *m;

}

std::string Timestamp::str() const {

  char* p;
  std::time_t t = ftime;
  if ((p = std::ctime(&t)) == 0) throw ESystem(EINVAL);

  // get rid of the newline, return the string

  if (char* q = std::strchr(p,'\n')) *q = 0;
  return std::string(p);

}

void Timestamp::decode(int& year, int& month, int& day, int& hour, int& min, int& sec) const {

  std::tm t;
  copyTo(t);

  year = t.tm_year+1900;
  month = ++t.tm_mon;
  day = t.tm_mday;
  hour = t.tm_hour;
  min = t.tm_min;
  sec = t.tm_sec;

}

int Timestamp::dow() const {

  // return value is:
  //
  //   1 = monday .. 7 = sunday
  //
  // according to ISO 8601

  std::tm t;
  copyTo(t);

  return t.tm_wday ? t.tm_wday : 7;

}

Timestamp now() {

  std::time_t t;
  if (std::time(&t) == (std::time_t)-1) throw ESystem(EINVAL);

  return Timestamp(t);

}

} // namespace IB
