/*******************************************************************************

Copyright (C) 2003 Andrea Biardi - LOGICOMM

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#include <cassert>
#include <libinterbasexx/database.h>
#include <libinterbasexx/parameterblock.h>

namespace IB {

Database::~Database() {

  // avoid throwing exceptions in the destructor; act silently.

  try {
    if (attached()) detach();
  }
  catch (const EInterbase&) {}

}

void Database::attach() {

  // make sure the database isn't already attached

  checkNotAttached();

  // set up a DPB (database parameter block)

  ParameterBlock dpb;
  dpb.push(isc_dpb_version1);
  if (!fusername.empty()) dpb.push(isc_dpb_user_name,fusername);
  if (!fpassword.empty()) dpb.push(isc_dpb_password,fpassword);
  if (!frole.empty()) dpb.push(isc_dpb_sql_role_name,frole);
  if (fbuffers) dpb.push(isc_dpb_num_buffers,fbuffers);

  // call isc_attach_database() to do the job

  ISC_STATUS v[20];
  if (isc_attach_database(v,0,(char*) fdatabase.c_str(),&fhandle,dpb.size(),dpb.data())) throw EInterbase(v);

  // we should now have a handle

  IBXX_ASSERT(fhandle);

}

void Database::detach() {

  // make sure the database is attached

  checkAttached();

  // call isc_detach_database() to do the job

  ISC_STATUS v[20];
  if (isc_detach_database(v,&fhandle)) throw EInterbase(v);

  // we should now have a null handle

  IBXX_ASSERT(!fhandle);

}

void Database::drop() {

  // make sure the database is attached

  checkAttached();

  // call isc_drop_database() to do the job; note that after the call, the
  // attachment is of course lost

  ISC_STATUS v[20];
  if (isc_drop_database(v,&fhandle)) throw EInterbase(v);

  // we should now have a null handle

  IBXX_ASSERT(!fhandle);

}

} // namespace IB
