/*******************************************************************************

Copyright (C) 2003 Andrea Biardi - LOGICOMM

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#ifndef __IBXX_DATABASE_H__
#define __IBXX_DATABASE_H__

/*******************************************************************************

Encapsulation of Interbase C API for database access.

*******************************************************************************/

#include <libinterbasexx/config.h>
#include <libinterbasexx/error.h>
#include <string>
#include <ibase.h>

namespace IB {

class IBXX_EXPORT Database {
  private:
    isc_db_handle fhandle;   // database handle
    std::string fdatabase;   // database name ("" = use environment variable ISC_DATABASE or ISC_PATH)
    std::string fusername;   // username ("" = use environment variable ISC_USER)
    std::string fpassword;   // password ("" = use environment variable ISC_PASSWORD)
    std::string frole;       // role
    unsigned char fbuffers;  // number of cache buffers (0 = default)
    unsigned short fdialect; // preferred dialect; just published here, actually not used by this class
  private:
    Database(const Database&) {}
    Database& operator=(const Database&) { return *this; }
  protected:
    void checkAttached() const { if (!attached()) throw EInterbase("Database is not attached"); }
    void checkNotAttached() const { if (attached()) throw EInterbase("Database is already attached"); }
  public:
    Database(const std::string& database = "", const std::string& username = "", const std::string& password = "", const std::string& role = ""): fhandle(0), fdatabase(database), fusername(username), fpassword(password), frole(role), fbuffers(0), fdialect(IBXX_DEFAULT_DIALECT) {}
    ~Database();
    isc_db_handle handle() const { checkAttached(); return fhandle; }
    const std::string& database() const { return fdatabase; }
    void setDatabase(const std::string& value) { checkNotAttached(); fdatabase = value; }
    const std::string& username() const { return fusername; }
    void setUsername(const std::string& value) { checkNotAttached(); fusername = value; }
    const std::string& password() const { return fpassword; }
    void setPassword(const std::string& value) { checkNotAttached(); fpassword = value; }
    const std::string& role() const { return frole; }
    void setRole(const std::string& value) { checkNotAttached(); frole = value; }
    unsigned char buffers() const { return fbuffers; }
    void setBuffers(unsigned char value) { checkNotAttached(); fbuffers = value; }
    unsigned short dialect() const { return fdialect; }
    void setDialect(unsigned short value) { checkNotAttached(); fdialect = value; }
    bool attached() const { return fhandle; }
    void attach();
    void detach();
    void drop();
};

} // namespace IB

#endif // __IBXX_DATABASE_H__
