/*******************************************************************************

Copyright (C) 2003 Andrea Biardi - LOGICOMM

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#include <cassert>
#include <libinterbasexx/error.h>

namespace IB {

EInterbase::EInterbase(ISC_STATUS status[]) {

  // make sure there's really an error condition

  IBXX_ASSERT(status[0] == 1);
  IBXX_ASSERT(status[1]);

#ifdef IBXX_ERROR_MSGS_SQL

  // SQL-style error messages are formatted here

  char buf[2048];

  ISC_LONG sqlcode = isc_sqlcode(status);
  isc_sql_interprete(sqlcode,buf,sizeof buf);

  setMsg(format("%s (SQL code: %d)",buf,sqlcode));

#else

  // hope the buffer is big enough, since isc_interprete() does not take its
  // size as an argument; however, isc_interprete() returns one line at a time,
  // so it should be ok even for very long error messages

  char buf[1024];

  isc_interprete(buf,&status);
  setMsg(buf);

  buf[0] = '\n';
  buf[1] = '-';
  while (isc_interprete(buf+2,&status)) setMsg(msg()+buf);

#endif

}

} // namespace IB
