/*******************************************************************************

Copyright (C) 2003 Andrea Biardi - LOGICOMM

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#ifndef __IBXX_INFO_H__
#define __IBXX_INFO_H__

/*******************************************************************************

Information retrieval for databases and statements/queries.

*******************************************************************************/

#include <libinterbasexx/config.h>
#include <libinterbasexx/database.h>
#include <libinterbasexx/query.h>
#include <libinterbasexx/statement.h>
#include <string>
#include <vector>
#include <ibase.h>

namespace IB {

class IBXX_EXPORT DatabaseInfo {
  public:
    enum Connections {
      Local  = 0x0002, // local connection
      Remote = 0x0004  // remote connection
    };
  private:
    std::vector<std::string> fdesc;  // description strings
    int flevel;                      // version (6 for interbase 6.0)
    std::string fversion;            // full server version string
    int fconnection;                 // connection type, see above
    std::string fhost;               // hostname (my.local.server)
    std::string fname;               // database name (/opt/interbase/examples/employee.gdb)
    int fodsMajor;                   // on-disk structure version (major)
    int fodsMinor;                   // on-disk structure version (minor)
    int fdialect;                    // dialect
    int fpageSize;                   // page size, in bytes
    int fpages;                      // number of database pages
    int fbuffers;                    // number of cache buffers
    int fusedMemory;                 // amount of memory currently used, in bytes
    std::vector<std::string> fusers; // users currently attached to the database
  protected:
    void getInfo(isc_db_handle handle);
    static isc_callback pushDesc(DatabaseInfo* n, char* c);
  public:
    DatabaseInfo(isc_db_handle handle) { getInfo(handle); }
    DatabaseInfo(const Database& database) { getInfo(database.handle()); }
    const std::vector<std::string>& desc() const { return fdesc; }
    std::string desc_1() const { return fdesc.empty() ? std::string("") : fdesc[0]; }
    int level() const { return flevel; }
    const std::string& version() const { return fversion; }
    int connection() const { return fconnection; }
    const std::string& host() const { return fhost; }
    const std::string& name() const { return fname; }
    int odsMajor() const { return fodsMajor; }
    int odsMinor() const { return fodsMinor; }
    int dialect() const { return fdialect; }
    int pageSize() const { return fpageSize; }
    int pages() const { return fpages; }
    int buffers() const { return fbuffers; }
    int usedMemory() const { return fusedMemory; }
    const std::vector<std::string>& users() const { return fusers; }
};

class IBXX_EXPORT StatementInfo {
  private:
    int fupdates; // number of updated rows
    int fdeletes; // number of deleted rows
    int fselects; // number of selected rows
    int finserts; // number of inserted rows
  protected:
    void getInfo(isc_stmt_handle handle);
  public:
    StatementInfo(isc_stmt_handle handle) { getInfo(handle); }
    StatementInfo(const Statement& statement) { getInfo(statement.handle()); }
    StatementInfo(const Query& query) { getInfo(query.handle()); }
    int updates() const { return fupdates; }
    int deletes() const { return fdeletes; }
    int selects() const { return fselects; }
    int inserts() const { return finserts; }
};

} // namespace IB

#endif // __IBXX_INFO_H__
