/*******************************************************************************

Copyright (C) 2003 Andrea Biardi - LOGICOMM

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#ifndef __IBXX_PARAMETERBLOCK_H__
#define __IBXX_PARAMETERBLOCK_H__

/*******************************************************************************

Base class for parameter blocks (size is limited to SHRT_MAX bytes).

*******************************************************************************/

#include <algorithm>
#include <libinterbasexx/config.h>
#include <libinterbasexx/cxxbits/cbuffer.h>
#include <string>

namespace IB {

class IBXX_EXPORT ParameterBlock {
  private:
    CBuffer fbuffer; // buffer (note that fbuffer.size() == allocated capacity)
    short fsize;     // (used) size
  protected:
    void needCapacity(int value);
  public:
    ParameterBlock(short capacity = 256): fbuffer(std::min(IBXX_ALIGN_SIZE(capacity),SHRT_MAX)), fsize(0) {}
    short capacity() const { return short(fbuffer.size()); }
    short size() const { return fsize; }
    bool empty() { return !fsize; }
    void clear() { fsize = 0; }
    const char* data() const { return (const char*) fbuffer.data(); }
    char* data() { return (char*) fbuffer.data(); }
    void push(char field);
    void push(char field, unsigned char value);
    void push(char field, const std::string& value);
};

inline void ParameterBlock::push(char field) {
  needCapacity(fsize+1);
  data()[fsize++] = field;
}

inline void ParameterBlock::push(char field, unsigned char value) {
  needCapacity(fsize+3);
  data()[fsize++] = field;
  data()[fsize++] = 1;
  data()[fsize++] = value;
}

inline void ParameterBlock::push(char field, const std::string& value) {
  if (unsigned char n = (unsigned char)(std::min(value.length(),std::string::size_type(255)))) {
    needCapacity(fsize+2+n);
    data()[fsize++] = field;
    data()[fsize++] = n;
    value.copy(data()+fsize,n);
    fsize += n;
  }
}

} // namespace IB

#endif // __IBXX_PARAMETERBLOCK_H__
