/*******************************************************************************

Copyright (C) 2003 Andrea Biardi - LOGICOMM

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#ifndef __IBXX_QUERY_H__
#define __IBXX_QUERY_H__

/*******************************************************************************

Encapsulation of Interbase C API for queries.

*******************************************************************************/

#include <libinterbasexx/config.h>
#include <libinterbasexx/cxxbits/cbuffer.h>
#include <libinterbasexx/database.h>
#include <libinterbasexx/error.h>
#include <libinterbasexx/sqloutvar.h>
#include <libinterbasexx/statement.h>
#include <libinterbasexx/transaction.h>
#include <libinterbasexx/xsqlda.h>
#include <string>
#include <ibase.h>

namespace IB {

class IBXX_EXPORT Query {
  protected:
    enum States {
      Prepared = 0x0001, // query is prepared
      Active   = 0x0002  // query is active (open)
    };
  public:
    class Out {
      friend class Query;
      private:
        CBuffer fxsqlda; // output XSQLDA
        CBuffer fvars;   // output buffer for SQL variables
      private:
        Out(const Out&) {}
        Out& operator=(const Out&) { return *this; }
      protected:
        void resize(short value);
        void prepare() { prepareXSQLVarsBuffer(xsqlda(),fvars); }
      public:
        Out(short count = 0): fvars(1024) { resize(count); }
        const XSQLDA* xsqlda() const { return (const XSQLDA*) fxsqlda.data(); }
        XSQLDA* xsqlda() { return (XSQLDA*) fxsqlda.data(); }
        short count() const { return xsqlda()->sqld; }
        SqlOutVar operator[](short index) const { return SqlOutVar((XSQLVAR*) xsqlda()->sqlvar+index); }
    };
  private:
    Statement fstatement; // statement
    int fstate;           // state, see above
    Out fout;             // query output
  private:
    Query(const Query& src): fstatement(src.database(),src.transaction()), fout(0) {}
    Query& operator=(const Query&) { return *this; }
  protected:
    void notPrepared() const;
    void alreadyPrepared() const;
    void checkPrepared() const { if (!prepared()) throw EInterbase("Query is not prepared"); }
    void checkNotPrepared() const { if (prepared()) throw EInterbase("Query is already prepared"); }
    void checkActive() const { if (!active()) throw EInterbase("Query is not active"); }
    void checkNotActive() const { if (active()) throw EInterbase("Query is already active"); }
  public:
    Query(const Database& database, const Transaction& transaction, const std::string& text = ""): fstatement(database,transaction,text), fstate(0), fout(32) {}
    ~Query();
    const Database& database() const { return fstatement.database(); }
    const Transaction& transaction() const { return fstatement.transaction(); }
    const std::string& text() const { return fstatement.text(); }
    void setText(const std::string& value) { checkNotActive(); fstate &= ~Prepared; fstatement.setText(value); }
    isc_stmt_handle handle() const { return fstatement.handle(); }
    bool prepared() const { return fstate & Prepared; }
    void prepare();
    bool active() const { return fstate & Active; }
    void open();
    void close();
    bool fetch();
    const Statement::In& in() const { return fstatement.in(); }
    Statement::In& in() { return fstatement.in(); }
    const Out& out() const { checkPrepared(); return fout; }
};

} // namespace IB

#endif // __IBXX_QUERY_H__
