/*******************************************************************************

Copyright (C) 2003 Andrea Biardi - LOGICOMM

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

*******************************************************************************/

#ifndef __IBXX_SQLINVAR_H__
#define __IBXX_SQLINVAR_H__

/*******************************************************************************

Encapsulation of SQL variables for input parameters.

*******************************************************************************/

#include <iostream>
#include <libinterbasexx/array.h>
#include <libinterbasexx/blob.h>
#include <libinterbasexx/config.h>
#include <libinterbasexx/cxxbits/timestamp.h>
#include <libinterbasexx/numeric.h>
#include <libinterbasexx/sqlvar.h>
#include <string>
#include <ibase.h>

namespace IB {

class IBXX_EXPORT SqlInVar: public SqlVar {
  private:
    int ftype;            // type (0 when null)
    IB::Numeric fvalue_n; // value, when ftype == SqlVar::Numeric
    Timestamp fvalue_t;   // value, when ftype == SqlVar::Time
    std::string fvalue_s; // value, when ftype == SqlVar::String
    IB::Array fvalue_a;   // value, when ftype == SqlVar::Array
    IB::Blob fvalue_b;    // value, when ftype == SqlVar::Blob
  protected:
    void copy(const SqlInVar& src);
    void set_n(const IB::Numeric& value) { ftype = Numeric; fvalue_n = value; }
    void set_t(const Timestamp& value) { ftype = Time; fvalue_t = value; }
    void set_s(const std::string& value) { ftype = String; fvalue_s = value; }
    void set_a(const IB::Array& value) { ftype = Array; fvalue_a = value; }
    void set_b(const IB::Blob& value) { ftype = Blob; fvalue_b = value; }
  public:
    SqlInVar(): ftype(0) {}
    SqlInVar(const IB::Numeric& value) { set_n(value); }
    SqlInVar(const Timestamp& value) { set_t(value); }
    SqlInVar(const std::string& value) { set_s(value); }
    SqlInVar(const IB::Array& value) { set_a(value); }
    SqlInVar(const IB::Blob& value) { set_b(value); }
    SqlInVar(const SqlInVar& src) { copy(src); }
    int type() const { return ftype; }
    bool null() const { return !ftype; }
    void clear() { ftype = 0; }
    const IB::Numeric& n() const;
    const Timestamp& t() const;
    const std::string& s() const;
    const IB::Array& a() const;
    const IB::Blob& b() const;
    void copyToXSQLVAR(XSQLVAR* v) const;
    SqlInVar& operator=(const IB::Numeric& value) { set_n(value); return *this; }
    SqlInVar& operator=(const Timestamp& value) { set_t(value); return *this; }
    SqlInVar& operator=(const std::string& value) { set_s(value); return *this; }
    SqlInVar& operator=(const IB::Array& value) { set_a(value); return *this; }
    SqlInVar& operator=(const IB::Blob& value) { set_b(value); return *this; }
    SqlInVar& operator=(const SqlInVar& value) { copy(value); return *this; }
    std::ostream& put(std::ostream& stream) const { return SqlVar::put(stream,*this); }
};

inline std::ostream& operator<<(std::ostream& stream, const SqlInVar& v) {
  return v.put(stream);
}

} // namespace IB

#endif // __IBXX_SQLINVAR_H__
